# coding=utf-8
# *** WARNING: this file was generated by the Pulumi Terraform Bridge (tfgen) Tool. ***
# *** Do not edit by hand unless you're certain you know what you are doing! ***

import builtins
import copy
import warnings
import sys
import pulumi
import pulumi.runtime
from typing import Any, Mapping, Optional, Sequence, Union, overload
if sys.version_info >= (3, 11):
    from typing import NotRequired, TypedDict, TypeAlias
else:
    from typing_extensions import NotRequired, TypedDict, TypeAlias
from . import _utilities
from . import outputs
from ._inputs import *

__all__ = ['ReleaseArgs', 'Release']

@pulumi.input_type
class ReleaseArgs:
    def __init__(__self__, *,
                 project: pulumi.Input[builtins.str],
                 tag_name: pulumi.Input[builtins.str],
                 assets: Optional[pulumi.Input['ReleaseAssetsArgs']] = None,
                 description: Optional[pulumi.Input[builtins.str]] = None,
                 milestones: Optional[pulumi.Input[Sequence[pulumi.Input[builtins.str]]]] = None,
                 name: Optional[pulumi.Input[builtins.str]] = None,
                 ref: Optional[pulumi.Input[builtins.str]] = None,
                 released_at: Optional[pulumi.Input[builtins.str]] = None,
                 tag_message: Optional[pulumi.Input[builtins.str]] = None):
        """
        The set of arguments for constructing a Release resource.
        :param pulumi.Input[builtins.str] project: The ID or full path of the project.
        :param pulumi.Input[builtins.str] tag_name: The tag where the release is created from.
        :param pulumi.Input['ReleaseAssetsArgs'] assets: The release assets.
        :param pulumi.Input[builtins.str] description: The description of the release. You can use Markdown.
        :param pulumi.Input[Sequence[pulumi.Input[builtins.str]]] milestones: The title of each milestone the release is associated with. GitLab Premium customers can specify group milestones.
        :param pulumi.Input[builtins.str] name: The name of the release.
        :param pulumi.Input[builtins.str] ref: If a tag specified in tag*name doesn't exist, the release is created from ref and tagged with tag*name. It can be a commit SHA, another tag name, or a branch name.
        :param pulumi.Input[builtins.str] released_at: Date and time for the release. Defaults to the current time. Expected in ISO 8601 format (2019-03-15T08:00:00Z). Only provide this field if creating an upcoming or historical release.
        :param pulumi.Input[builtins.str] tag_message: Message to use if creating a new annotated tag.
        """
        pulumi.set(__self__, "project", project)
        pulumi.set(__self__, "tag_name", tag_name)
        if assets is not None:
            pulumi.set(__self__, "assets", assets)
        if description is not None:
            pulumi.set(__self__, "description", description)
        if milestones is not None:
            pulumi.set(__self__, "milestones", milestones)
        if name is not None:
            pulumi.set(__self__, "name", name)
        if ref is not None:
            pulumi.set(__self__, "ref", ref)
        if released_at is not None:
            pulumi.set(__self__, "released_at", released_at)
        if tag_message is not None:
            pulumi.set(__self__, "tag_message", tag_message)

    @property
    @pulumi.getter
    def project(self) -> pulumi.Input[builtins.str]:
        """
        The ID or full path of the project.
        """
        return pulumi.get(self, "project")

    @project.setter
    def project(self, value: pulumi.Input[builtins.str]):
        pulumi.set(self, "project", value)

    @property
    @pulumi.getter(name="tagName")
    def tag_name(self) -> pulumi.Input[builtins.str]:
        """
        The tag where the release is created from.
        """
        return pulumi.get(self, "tag_name")

    @tag_name.setter
    def tag_name(self, value: pulumi.Input[builtins.str]):
        pulumi.set(self, "tag_name", value)

    @property
    @pulumi.getter
    def assets(self) -> Optional[pulumi.Input['ReleaseAssetsArgs']]:
        """
        The release assets.
        """
        return pulumi.get(self, "assets")

    @assets.setter
    def assets(self, value: Optional[pulumi.Input['ReleaseAssetsArgs']]):
        pulumi.set(self, "assets", value)

    @property
    @pulumi.getter
    def description(self) -> Optional[pulumi.Input[builtins.str]]:
        """
        The description of the release. You can use Markdown.
        """
        return pulumi.get(self, "description")

    @description.setter
    def description(self, value: Optional[pulumi.Input[builtins.str]]):
        pulumi.set(self, "description", value)

    @property
    @pulumi.getter
    def milestones(self) -> Optional[pulumi.Input[Sequence[pulumi.Input[builtins.str]]]]:
        """
        The title of each milestone the release is associated with. GitLab Premium customers can specify group milestones.
        """
        return pulumi.get(self, "milestones")

    @milestones.setter
    def milestones(self, value: Optional[pulumi.Input[Sequence[pulumi.Input[builtins.str]]]]):
        pulumi.set(self, "milestones", value)

    @property
    @pulumi.getter
    def name(self) -> Optional[pulumi.Input[builtins.str]]:
        """
        The name of the release.
        """
        return pulumi.get(self, "name")

    @name.setter
    def name(self, value: Optional[pulumi.Input[builtins.str]]):
        pulumi.set(self, "name", value)

    @property
    @pulumi.getter
    def ref(self) -> Optional[pulumi.Input[builtins.str]]:
        """
        If a tag specified in tag*name doesn't exist, the release is created from ref and tagged with tag*name. It can be a commit SHA, another tag name, or a branch name.
        """
        return pulumi.get(self, "ref")

    @ref.setter
    def ref(self, value: Optional[pulumi.Input[builtins.str]]):
        pulumi.set(self, "ref", value)

    @property
    @pulumi.getter(name="releasedAt")
    def released_at(self) -> Optional[pulumi.Input[builtins.str]]:
        """
        Date and time for the release. Defaults to the current time. Expected in ISO 8601 format (2019-03-15T08:00:00Z). Only provide this field if creating an upcoming or historical release.
        """
        return pulumi.get(self, "released_at")

    @released_at.setter
    def released_at(self, value: Optional[pulumi.Input[builtins.str]]):
        pulumi.set(self, "released_at", value)

    @property
    @pulumi.getter(name="tagMessage")
    def tag_message(self) -> Optional[pulumi.Input[builtins.str]]:
        """
        Message to use if creating a new annotated tag.
        """
        return pulumi.get(self, "tag_message")

    @tag_message.setter
    def tag_message(self, value: Optional[pulumi.Input[builtins.str]]):
        pulumi.set(self, "tag_message", value)


@pulumi.input_type
class _ReleaseState:
    def __init__(__self__, *,
                 assets: Optional[pulumi.Input['ReleaseAssetsArgs']] = None,
                 author: Optional[pulumi.Input['ReleaseAuthorArgs']] = None,
                 commit: Optional[pulumi.Input['ReleaseCommitArgs']] = None,
                 commit_path: Optional[pulumi.Input[builtins.str]] = None,
                 created_at: Optional[pulumi.Input[builtins.str]] = None,
                 description: Optional[pulumi.Input[builtins.str]] = None,
                 description_html: Optional[pulumi.Input[builtins.str]] = None,
                 links: Optional[pulumi.Input['ReleaseLinksArgs']] = None,
                 milestones: Optional[pulumi.Input[Sequence[pulumi.Input[builtins.str]]]] = None,
                 name: Optional[pulumi.Input[builtins.str]] = None,
                 project: Optional[pulumi.Input[builtins.str]] = None,
                 ref: Optional[pulumi.Input[builtins.str]] = None,
                 released_at: Optional[pulumi.Input[builtins.str]] = None,
                 tag_message: Optional[pulumi.Input[builtins.str]] = None,
                 tag_name: Optional[pulumi.Input[builtins.str]] = None,
                 tag_path: Optional[pulumi.Input[builtins.str]] = None,
                 upcoming_release: Optional[pulumi.Input[builtins.bool]] = None):
        """
        Input properties used for looking up and filtering Release resources.
        :param pulumi.Input['ReleaseAssetsArgs'] assets: The release assets.
        :param pulumi.Input['ReleaseAuthorArgs'] author: The author of the release.
        :param pulumi.Input['ReleaseCommitArgs'] commit: The release commit.
        :param pulumi.Input[builtins.str] commit_path: The path to the commit
        :param pulumi.Input[builtins.str] created_at: Date and time the release was created. In ISO 8601 format (2019-03-15T08:00:00Z).
        :param pulumi.Input[builtins.str] description: The description of the release. You can use Markdown.
        :param pulumi.Input[builtins.str] description_html: HTML rendered Markdown of the release description.
        :param pulumi.Input['ReleaseLinksArgs'] links: Links of the release
        :param pulumi.Input[Sequence[pulumi.Input[builtins.str]]] milestones: The title of each milestone the release is associated with. GitLab Premium customers can specify group milestones.
        :param pulumi.Input[builtins.str] name: The name of the release.
        :param pulumi.Input[builtins.str] project: The ID or full path of the project.
        :param pulumi.Input[builtins.str] ref: If a tag specified in tag*name doesn't exist, the release is created from ref and tagged with tag*name. It can be a commit SHA, another tag name, or a branch name.
        :param pulumi.Input[builtins.str] released_at: Date and time for the release. Defaults to the current time. Expected in ISO 8601 format (2019-03-15T08:00:00Z). Only provide this field if creating an upcoming or historical release.
        :param pulumi.Input[builtins.str] tag_message: Message to use if creating a new annotated tag.
        :param pulumi.Input[builtins.str] tag_name: The tag where the release is created from.
        :param pulumi.Input[builtins.str] tag_path: The path to the tag.
        :param pulumi.Input[builtins.bool] upcoming_release: Whether the release_at attribute is set to a future date.
        """
        if assets is not None:
            pulumi.set(__self__, "assets", assets)
        if author is not None:
            pulumi.set(__self__, "author", author)
        if commit is not None:
            pulumi.set(__self__, "commit", commit)
        if commit_path is not None:
            pulumi.set(__self__, "commit_path", commit_path)
        if created_at is not None:
            pulumi.set(__self__, "created_at", created_at)
        if description is not None:
            pulumi.set(__self__, "description", description)
        if description_html is not None:
            pulumi.set(__self__, "description_html", description_html)
        if links is not None:
            pulumi.set(__self__, "links", links)
        if milestones is not None:
            pulumi.set(__self__, "milestones", milestones)
        if name is not None:
            pulumi.set(__self__, "name", name)
        if project is not None:
            pulumi.set(__self__, "project", project)
        if ref is not None:
            pulumi.set(__self__, "ref", ref)
        if released_at is not None:
            pulumi.set(__self__, "released_at", released_at)
        if tag_message is not None:
            pulumi.set(__self__, "tag_message", tag_message)
        if tag_name is not None:
            pulumi.set(__self__, "tag_name", tag_name)
        if tag_path is not None:
            pulumi.set(__self__, "tag_path", tag_path)
        if upcoming_release is not None:
            pulumi.set(__self__, "upcoming_release", upcoming_release)

    @property
    @pulumi.getter
    def assets(self) -> Optional[pulumi.Input['ReleaseAssetsArgs']]:
        """
        The release assets.
        """
        return pulumi.get(self, "assets")

    @assets.setter
    def assets(self, value: Optional[pulumi.Input['ReleaseAssetsArgs']]):
        pulumi.set(self, "assets", value)

    @property
    @pulumi.getter
    def author(self) -> Optional[pulumi.Input['ReleaseAuthorArgs']]:
        """
        The author of the release.
        """
        return pulumi.get(self, "author")

    @author.setter
    def author(self, value: Optional[pulumi.Input['ReleaseAuthorArgs']]):
        pulumi.set(self, "author", value)

    @property
    @pulumi.getter
    def commit(self) -> Optional[pulumi.Input['ReleaseCommitArgs']]:
        """
        The release commit.
        """
        return pulumi.get(self, "commit")

    @commit.setter
    def commit(self, value: Optional[pulumi.Input['ReleaseCommitArgs']]):
        pulumi.set(self, "commit", value)

    @property
    @pulumi.getter(name="commitPath")
    def commit_path(self) -> Optional[pulumi.Input[builtins.str]]:
        """
        The path to the commit
        """
        return pulumi.get(self, "commit_path")

    @commit_path.setter
    def commit_path(self, value: Optional[pulumi.Input[builtins.str]]):
        pulumi.set(self, "commit_path", value)

    @property
    @pulumi.getter(name="createdAt")
    def created_at(self) -> Optional[pulumi.Input[builtins.str]]:
        """
        Date and time the release was created. In ISO 8601 format (2019-03-15T08:00:00Z).
        """
        return pulumi.get(self, "created_at")

    @created_at.setter
    def created_at(self, value: Optional[pulumi.Input[builtins.str]]):
        pulumi.set(self, "created_at", value)

    @property
    @pulumi.getter
    def description(self) -> Optional[pulumi.Input[builtins.str]]:
        """
        The description of the release. You can use Markdown.
        """
        return pulumi.get(self, "description")

    @description.setter
    def description(self, value: Optional[pulumi.Input[builtins.str]]):
        pulumi.set(self, "description", value)

    @property
    @pulumi.getter(name="descriptionHtml")
    def description_html(self) -> Optional[pulumi.Input[builtins.str]]:
        """
        HTML rendered Markdown of the release description.
        """
        return pulumi.get(self, "description_html")

    @description_html.setter
    def description_html(self, value: Optional[pulumi.Input[builtins.str]]):
        pulumi.set(self, "description_html", value)

    @property
    @pulumi.getter
    def links(self) -> Optional[pulumi.Input['ReleaseLinksArgs']]:
        """
        Links of the release
        """
        return pulumi.get(self, "links")

    @links.setter
    def links(self, value: Optional[pulumi.Input['ReleaseLinksArgs']]):
        pulumi.set(self, "links", value)

    @property
    @pulumi.getter
    def milestones(self) -> Optional[pulumi.Input[Sequence[pulumi.Input[builtins.str]]]]:
        """
        The title of each milestone the release is associated with. GitLab Premium customers can specify group milestones.
        """
        return pulumi.get(self, "milestones")

    @milestones.setter
    def milestones(self, value: Optional[pulumi.Input[Sequence[pulumi.Input[builtins.str]]]]):
        pulumi.set(self, "milestones", value)

    @property
    @pulumi.getter
    def name(self) -> Optional[pulumi.Input[builtins.str]]:
        """
        The name of the release.
        """
        return pulumi.get(self, "name")

    @name.setter
    def name(self, value: Optional[pulumi.Input[builtins.str]]):
        pulumi.set(self, "name", value)

    @property
    @pulumi.getter
    def project(self) -> Optional[pulumi.Input[builtins.str]]:
        """
        The ID or full path of the project.
        """
        return pulumi.get(self, "project")

    @project.setter
    def project(self, value: Optional[pulumi.Input[builtins.str]]):
        pulumi.set(self, "project", value)

    @property
    @pulumi.getter
    def ref(self) -> Optional[pulumi.Input[builtins.str]]:
        """
        If a tag specified in tag*name doesn't exist, the release is created from ref and tagged with tag*name. It can be a commit SHA, another tag name, or a branch name.
        """
        return pulumi.get(self, "ref")

    @ref.setter
    def ref(self, value: Optional[pulumi.Input[builtins.str]]):
        pulumi.set(self, "ref", value)

    @property
    @pulumi.getter(name="releasedAt")
    def released_at(self) -> Optional[pulumi.Input[builtins.str]]:
        """
        Date and time for the release. Defaults to the current time. Expected in ISO 8601 format (2019-03-15T08:00:00Z). Only provide this field if creating an upcoming or historical release.
        """
        return pulumi.get(self, "released_at")

    @released_at.setter
    def released_at(self, value: Optional[pulumi.Input[builtins.str]]):
        pulumi.set(self, "released_at", value)

    @property
    @pulumi.getter(name="tagMessage")
    def tag_message(self) -> Optional[pulumi.Input[builtins.str]]:
        """
        Message to use if creating a new annotated tag.
        """
        return pulumi.get(self, "tag_message")

    @tag_message.setter
    def tag_message(self, value: Optional[pulumi.Input[builtins.str]]):
        pulumi.set(self, "tag_message", value)

    @property
    @pulumi.getter(name="tagName")
    def tag_name(self) -> Optional[pulumi.Input[builtins.str]]:
        """
        The tag where the release is created from.
        """
        return pulumi.get(self, "tag_name")

    @tag_name.setter
    def tag_name(self, value: Optional[pulumi.Input[builtins.str]]):
        pulumi.set(self, "tag_name", value)

    @property
    @pulumi.getter(name="tagPath")
    def tag_path(self) -> Optional[pulumi.Input[builtins.str]]:
        """
        The path to the tag.
        """
        return pulumi.get(self, "tag_path")

    @tag_path.setter
    def tag_path(self, value: Optional[pulumi.Input[builtins.str]]):
        pulumi.set(self, "tag_path", value)

    @property
    @pulumi.getter(name="upcomingRelease")
    def upcoming_release(self) -> Optional[pulumi.Input[builtins.bool]]:
        """
        Whether the release_at attribute is set to a future date.
        """
        return pulumi.get(self, "upcoming_release")

    @upcoming_release.setter
    def upcoming_release(self, value: Optional[pulumi.Input[builtins.bool]]):
        pulumi.set(self, "upcoming_release", value)


class Release(pulumi.CustomResource):

    pulumi_type = "gitlab:index/release:Release"

    @overload
    def __init__(__self__,
                 resource_name: str,
                 opts: Optional[pulumi.ResourceOptions] = None,
                 assets: Optional[pulumi.Input[Union['ReleaseAssetsArgs', 'ReleaseAssetsArgsDict']]] = None,
                 description: Optional[pulumi.Input[builtins.str]] = None,
                 milestones: Optional[pulumi.Input[Sequence[pulumi.Input[builtins.str]]]] = None,
                 name: Optional[pulumi.Input[builtins.str]] = None,
                 project: Optional[pulumi.Input[builtins.str]] = None,
                 ref: Optional[pulumi.Input[builtins.str]] = None,
                 released_at: Optional[pulumi.Input[builtins.str]] = None,
                 tag_message: Optional[pulumi.Input[builtins.str]] = None,
                 tag_name: Optional[pulumi.Input[builtins.str]] = None,
                 __props__=None):
        """
        The `Release` resource allows to manage the lifecycle of releases in gitlab.

        **Upstream API**: [GitLab REST API docs](https://docs.gitlab.com/api/releases/)

        ## Example Usage

        ```python
        import pulumi
        import pulumi_gitlab as gitlab

        # Create a project
        example = gitlab.Project("example",
            name="example",
            description="An example project")
        # Create a release
        example_release = gitlab.Release("example",
            project=example.id,
            name="test-release",
            tag_name="v1.0.0",
            description="Test release description",
            ref="main")
        ```

        ## Import

        Starting in Terraform v1.5.0 you can use an import block to import `gitlab_release`. For example:

        terraform

        import {

          to = gitlab_release.example

          id = "see CLI command below for ID"

        }

        Import using the CLI is supported using the following syntax:

        Gitlab release link can be imported with a key composed of `<project>:<tag_name>`, e.g.

        ```sh
        $ pulumi import gitlab:index/release:Release example "12345:test"
        ```

        :param str resource_name: The name of the resource.
        :param pulumi.ResourceOptions opts: Options for the resource.
        :param pulumi.Input[Union['ReleaseAssetsArgs', 'ReleaseAssetsArgsDict']] assets: The release assets.
        :param pulumi.Input[builtins.str] description: The description of the release. You can use Markdown.
        :param pulumi.Input[Sequence[pulumi.Input[builtins.str]]] milestones: The title of each milestone the release is associated with. GitLab Premium customers can specify group milestones.
        :param pulumi.Input[builtins.str] name: The name of the release.
        :param pulumi.Input[builtins.str] project: The ID or full path of the project.
        :param pulumi.Input[builtins.str] ref: If a tag specified in tag*name doesn't exist, the release is created from ref and tagged with tag*name. It can be a commit SHA, another tag name, or a branch name.
        :param pulumi.Input[builtins.str] released_at: Date and time for the release. Defaults to the current time. Expected in ISO 8601 format (2019-03-15T08:00:00Z). Only provide this field if creating an upcoming or historical release.
        :param pulumi.Input[builtins.str] tag_message: Message to use if creating a new annotated tag.
        :param pulumi.Input[builtins.str] tag_name: The tag where the release is created from.
        """
        ...
    @overload
    def __init__(__self__,
                 resource_name: str,
                 args: ReleaseArgs,
                 opts: Optional[pulumi.ResourceOptions] = None):
        """
        The `Release` resource allows to manage the lifecycle of releases in gitlab.

        **Upstream API**: [GitLab REST API docs](https://docs.gitlab.com/api/releases/)

        ## Example Usage

        ```python
        import pulumi
        import pulumi_gitlab as gitlab

        # Create a project
        example = gitlab.Project("example",
            name="example",
            description="An example project")
        # Create a release
        example_release = gitlab.Release("example",
            project=example.id,
            name="test-release",
            tag_name="v1.0.0",
            description="Test release description",
            ref="main")
        ```

        ## Import

        Starting in Terraform v1.5.0 you can use an import block to import `gitlab_release`. For example:

        terraform

        import {

          to = gitlab_release.example

          id = "see CLI command below for ID"

        }

        Import using the CLI is supported using the following syntax:

        Gitlab release link can be imported with a key composed of `<project>:<tag_name>`, e.g.

        ```sh
        $ pulumi import gitlab:index/release:Release example "12345:test"
        ```

        :param str resource_name: The name of the resource.
        :param ReleaseArgs args: The arguments to use to populate this resource's properties.
        :param pulumi.ResourceOptions opts: Options for the resource.
        """
        ...
    def __init__(__self__, resource_name: str, *args, **kwargs):
        resource_args, opts = _utilities.get_resource_args_opts(ReleaseArgs, pulumi.ResourceOptions, *args, **kwargs)
        if resource_args is not None:
            __self__._internal_init(resource_name, opts, **resource_args.__dict__)
        else:
            __self__._internal_init(resource_name, *args, **kwargs)

    def _internal_init(__self__,
                 resource_name: str,
                 opts: Optional[pulumi.ResourceOptions] = None,
                 assets: Optional[pulumi.Input[Union['ReleaseAssetsArgs', 'ReleaseAssetsArgsDict']]] = None,
                 description: Optional[pulumi.Input[builtins.str]] = None,
                 milestones: Optional[pulumi.Input[Sequence[pulumi.Input[builtins.str]]]] = None,
                 name: Optional[pulumi.Input[builtins.str]] = None,
                 project: Optional[pulumi.Input[builtins.str]] = None,
                 ref: Optional[pulumi.Input[builtins.str]] = None,
                 released_at: Optional[pulumi.Input[builtins.str]] = None,
                 tag_message: Optional[pulumi.Input[builtins.str]] = None,
                 tag_name: Optional[pulumi.Input[builtins.str]] = None,
                 __props__=None):
        opts = pulumi.ResourceOptions.merge(_utilities.get_resource_opts_defaults(), opts)
        if not isinstance(opts, pulumi.ResourceOptions):
            raise TypeError('Expected resource options to be a ResourceOptions instance')
        if opts.id is None:
            if __props__ is not None:
                raise TypeError('__props__ is only valid when passed in combination with a valid opts.id to get an existing resource')
            __props__ = ReleaseArgs.__new__(ReleaseArgs)

            __props__.__dict__["assets"] = assets
            __props__.__dict__["description"] = description
            __props__.__dict__["milestones"] = milestones
            __props__.__dict__["name"] = name
            if project is None and not opts.urn:
                raise TypeError("Missing required property 'project'")
            __props__.__dict__["project"] = project
            __props__.__dict__["ref"] = ref
            __props__.__dict__["released_at"] = released_at
            __props__.__dict__["tag_message"] = tag_message
            if tag_name is None and not opts.urn:
                raise TypeError("Missing required property 'tag_name'")
            __props__.__dict__["tag_name"] = tag_name
            __props__.__dict__["author"] = None
            __props__.__dict__["commit"] = None
            __props__.__dict__["commit_path"] = None
            __props__.__dict__["created_at"] = None
            __props__.__dict__["description_html"] = None
            __props__.__dict__["links"] = None
            __props__.__dict__["tag_path"] = None
            __props__.__dict__["upcoming_release"] = None
        super(Release, __self__).__init__(
            'gitlab:index/release:Release',
            resource_name,
            __props__,
            opts)

    @staticmethod
    def get(resource_name: str,
            id: pulumi.Input[str],
            opts: Optional[pulumi.ResourceOptions] = None,
            assets: Optional[pulumi.Input[Union['ReleaseAssetsArgs', 'ReleaseAssetsArgsDict']]] = None,
            author: Optional[pulumi.Input[Union['ReleaseAuthorArgs', 'ReleaseAuthorArgsDict']]] = None,
            commit: Optional[pulumi.Input[Union['ReleaseCommitArgs', 'ReleaseCommitArgsDict']]] = None,
            commit_path: Optional[pulumi.Input[builtins.str]] = None,
            created_at: Optional[pulumi.Input[builtins.str]] = None,
            description: Optional[pulumi.Input[builtins.str]] = None,
            description_html: Optional[pulumi.Input[builtins.str]] = None,
            links: Optional[pulumi.Input[Union['ReleaseLinksArgs', 'ReleaseLinksArgsDict']]] = None,
            milestones: Optional[pulumi.Input[Sequence[pulumi.Input[builtins.str]]]] = None,
            name: Optional[pulumi.Input[builtins.str]] = None,
            project: Optional[pulumi.Input[builtins.str]] = None,
            ref: Optional[pulumi.Input[builtins.str]] = None,
            released_at: Optional[pulumi.Input[builtins.str]] = None,
            tag_message: Optional[pulumi.Input[builtins.str]] = None,
            tag_name: Optional[pulumi.Input[builtins.str]] = None,
            tag_path: Optional[pulumi.Input[builtins.str]] = None,
            upcoming_release: Optional[pulumi.Input[builtins.bool]] = None) -> 'Release':
        """
        Get an existing Release resource's state with the given name, id, and optional extra
        properties used to qualify the lookup.

        :param str resource_name: The unique name of the resulting resource.
        :param pulumi.Input[str] id: The unique provider ID of the resource to lookup.
        :param pulumi.ResourceOptions opts: Options for the resource.
        :param pulumi.Input[Union['ReleaseAssetsArgs', 'ReleaseAssetsArgsDict']] assets: The release assets.
        :param pulumi.Input[Union['ReleaseAuthorArgs', 'ReleaseAuthorArgsDict']] author: The author of the release.
        :param pulumi.Input[Union['ReleaseCommitArgs', 'ReleaseCommitArgsDict']] commit: The release commit.
        :param pulumi.Input[builtins.str] commit_path: The path to the commit
        :param pulumi.Input[builtins.str] created_at: Date and time the release was created. In ISO 8601 format (2019-03-15T08:00:00Z).
        :param pulumi.Input[builtins.str] description: The description of the release. You can use Markdown.
        :param pulumi.Input[builtins.str] description_html: HTML rendered Markdown of the release description.
        :param pulumi.Input[Union['ReleaseLinksArgs', 'ReleaseLinksArgsDict']] links: Links of the release
        :param pulumi.Input[Sequence[pulumi.Input[builtins.str]]] milestones: The title of each milestone the release is associated with. GitLab Premium customers can specify group milestones.
        :param pulumi.Input[builtins.str] name: The name of the release.
        :param pulumi.Input[builtins.str] project: The ID or full path of the project.
        :param pulumi.Input[builtins.str] ref: If a tag specified in tag*name doesn't exist, the release is created from ref and tagged with tag*name. It can be a commit SHA, another tag name, or a branch name.
        :param pulumi.Input[builtins.str] released_at: Date and time for the release. Defaults to the current time. Expected in ISO 8601 format (2019-03-15T08:00:00Z). Only provide this field if creating an upcoming or historical release.
        :param pulumi.Input[builtins.str] tag_message: Message to use if creating a new annotated tag.
        :param pulumi.Input[builtins.str] tag_name: The tag where the release is created from.
        :param pulumi.Input[builtins.str] tag_path: The path to the tag.
        :param pulumi.Input[builtins.bool] upcoming_release: Whether the release_at attribute is set to a future date.
        """
        opts = pulumi.ResourceOptions.merge(opts, pulumi.ResourceOptions(id=id))

        __props__ = _ReleaseState.__new__(_ReleaseState)

        __props__.__dict__["assets"] = assets
        __props__.__dict__["author"] = author
        __props__.__dict__["commit"] = commit
        __props__.__dict__["commit_path"] = commit_path
        __props__.__dict__["created_at"] = created_at
        __props__.__dict__["description"] = description
        __props__.__dict__["description_html"] = description_html
        __props__.__dict__["links"] = links
        __props__.__dict__["milestones"] = milestones
        __props__.__dict__["name"] = name
        __props__.__dict__["project"] = project
        __props__.__dict__["ref"] = ref
        __props__.__dict__["released_at"] = released_at
        __props__.__dict__["tag_message"] = tag_message
        __props__.__dict__["tag_name"] = tag_name
        __props__.__dict__["tag_path"] = tag_path
        __props__.__dict__["upcoming_release"] = upcoming_release
        return Release(resource_name, opts=opts, __props__=__props__)

    @property
    @pulumi.getter
    def assets(self) -> pulumi.Output['outputs.ReleaseAssets']:
        """
        The release assets.
        """
        return pulumi.get(self, "assets")

    @property
    @pulumi.getter
    def author(self) -> pulumi.Output['outputs.ReleaseAuthor']:
        """
        The author of the release.
        """
        return pulumi.get(self, "author")

    @property
    @pulumi.getter
    def commit(self) -> pulumi.Output['outputs.ReleaseCommit']:
        """
        The release commit.
        """
        return pulumi.get(self, "commit")

    @property
    @pulumi.getter(name="commitPath")
    def commit_path(self) -> pulumi.Output[builtins.str]:
        """
        The path to the commit
        """
        return pulumi.get(self, "commit_path")

    @property
    @pulumi.getter(name="createdAt")
    def created_at(self) -> pulumi.Output[builtins.str]:
        """
        Date and time the release was created. In ISO 8601 format (2019-03-15T08:00:00Z).
        """
        return pulumi.get(self, "created_at")

    @property
    @pulumi.getter
    def description(self) -> pulumi.Output[builtins.str]:
        """
        The description of the release. You can use Markdown.
        """
        return pulumi.get(self, "description")

    @property
    @pulumi.getter(name="descriptionHtml")
    def description_html(self) -> pulumi.Output[builtins.str]:
        """
        HTML rendered Markdown of the release description.
        """
        return pulumi.get(self, "description_html")

    @property
    @pulumi.getter
    def links(self) -> pulumi.Output['outputs.ReleaseLinks']:
        """
        Links of the release
        """
        return pulumi.get(self, "links")

    @property
    @pulumi.getter
    def milestones(self) -> pulumi.Output[Optional[Sequence[builtins.str]]]:
        """
        The title of each milestone the release is associated with. GitLab Premium customers can specify group milestones.
        """
        return pulumi.get(self, "milestones")

    @property
    @pulumi.getter
    def name(self) -> pulumi.Output[builtins.str]:
        """
        The name of the release.
        """
        return pulumi.get(self, "name")

    @property
    @pulumi.getter
    def project(self) -> pulumi.Output[builtins.str]:
        """
        The ID or full path of the project.
        """
        return pulumi.get(self, "project")

    @property
    @pulumi.getter
    def ref(self) -> pulumi.Output[builtins.str]:
        """
        If a tag specified in tag*name doesn't exist, the release is created from ref and tagged with tag*name. It can be a commit SHA, another tag name, or a branch name.
        """
        return pulumi.get(self, "ref")

    @property
    @pulumi.getter(name="releasedAt")
    def released_at(self) -> pulumi.Output[builtins.str]:
        """
        Date and time for the release. Defaults to the current time. Expected in ISO 8601 format (2019-03-15T08:00:00Z). Only provide this field if creating an upcoming or historical release.
        """
        return pulumi.get(self, "released_at")

    @property
    @pulumi.getter(name="tagMessage")
    def tag_message(self) -> pulumi.Output[Optional[builtins.str]]:
        """
        Message to use if creating a new annotated tag.
        """
        return pulumi.get(self, "tag_message")

    @property
    @pulumi.getter(name="tagName")
    def tag_name(self) -> pulumi.Output[builtins.str]:
        """
        The tag where the release is created from.
        """
        return pulumi.get(self, "tag_name")

    @property
    @pulumi.getter(name="tagPath")
    def tag_path(self) -> pulumi.Output[builtins.str]:
        """
        The path to the tag.
        """
        return pulumi.get(self, "tag_path")

    @property
    @pulumi.getter(name="upcomingRelease")
    def upcoming_release(self) -> pulumi.Output[builtins.bool]:
        """
        Whether the release_at attribute is set to a future date.
        """
        return pulumi.get(self, "upcoming_release")




# coding=utf-8
# *** WARNING: this file was generated by the Pulumi Terraform Bridge (tfgen) Tool. ***
# *** Do not edit by hand unless you're certain you know what you are doing! ***

import builtins
import copy
import warnings
import sys
import pulumi
import pulumi.runtime
from typing import Any, Mapping, Optional, Sequence, Union, overload
if sys.version_info >= (3, 11):
    from typing import NotRequired, TypedDict, TypeAlias
else:
    from typing_extensions import NotRequired, TypedDict, TypeAlias
from . import _utilities
from . import outputs
from ._inputs import *

__all__ = ['BranchProtectionArgs', 'BranchProtection']

@pulumi.input_type
class BranchProtectionArgs:
    def __init__(__self__, *,
                 branch: pulumi.Input[builtins.str],
                 project: pulumi.Input[builtins.str],
                 allow_force_push: Optional[pulumi.Input[builtins.bool]] = None,
                 allowed_to_merges: Optional[pulumi.Input[Sequence[pulumi.Input['BranchProtectionAllowedToMergeArgs']]]] = None,
                 allowed_to_pushes: Optional[pulumi.Input[Sequence[pulumi.Input['BranchProtectionAllowedToPushArgs']]]] = None,
                 allowed_to_unprotects: Optional[pulumi.Input[Sequence[pulumi.Input['BranchProtectionAllowedToUnprotectArgs']]]] = None,
                 code_owner_approval_required: Optional[pulumi.Input[builtins.bool]] = None,
                 merge_access_level: Optional[pulumi.Input[builtins.str]] = None,
                 push_access_level: Optional[pulumi.Input[builtins.str]] = None,
                 unprotect_access_level: Optional[pulumi.Input[builtins.str]] = None):
        """
        The set of arguments for constructing a BranchProtection resource.
        :param pulumi.Input[builtins.str] branch: Name of the branch.
        :param pulumi.Input[builtins.str] project: The id of the project.
        :param pulumi.Input[builtins.bool] allow_force_push: Can be set to true to allow users with push access to force push.
        :param pulumi.Input[Sequence[pulumi.Input['BranchProtectionAllowedToMergeArgs']]] allowed_to_merges: Array of access levels and user(s)/group(s) allowed to merge to protected branch.
        :param pulumi.Input[Sequence[pulumi.Input['BranchProtectionAllowedToPushArgs']]] allowed_to_pushes: Array of access levels and user(s)/group(s) allowed to push to protected branch.
        :param pulumi.Input[Sequence[pulumi.Input['BranchProtectionAllowedToUnprotectArgs']]] allowed_to_unprotects: Array of access levels and user(s)/group(s) allowed to unprotect push to protected branch.
        :param pulumi.Input[builtins.bool] code_owner_approval_required: Can be set to true to require code owner approval before merging. Only available for Premium and Ultimate instances.
        :param pulumi.Input[builtins.str] merge_access_level: Access levels allowed to merge. Valid values are: `no one`, `developer`, `maintainer`.
        :param pulumi.Input[builtins.str] push_access_level: Access levels allowed to push. Valid values are: `no one`, `developer`, `maintainer`.
        :param pulumi.Input[builtins.str] unprotect_access_level: Access levels allowed to unprotect. Valid values are: `developer`, `maintainer`, `admin`.
        """
        pulumi.set(__self__, "branch", branch)
        pulumi.set(__self__, "project", project)
        if allow_force_push is not None:
            pulumi.set(__self__, "allow_force_push", allow_force_push)
        if allowed_to_merges is not None:
            pulumi.set(__self__, "allowed_to_merges", allowed_to_merges)
        if allowed_to_pushes is not None:
            pulumi.set(__self__, "allowed_to_pushes", allowed_to_pushes)
        if allowed_to_unprotects is not None:
            pulumi.set(__self__, "allowed_to_unprotects", allowed_to_unprotects)
        if code_owner_approval_required is not None:
            pulumi.set(__self__, "code_owner_approval_required", code_owner_approval_required)
        if merge_access_level is not None:
            pulumi.set(__self__, "merge_access_level", merge_access_level)
        if push_access_level is not None:
            pulumi.set(__self__, "push_access_level", push_access_level)
        if unprotect_access_level is not None:
            pulumi.set(__self__, "unprotect_access_level", unprotect_access_level)

    @property
    @pulumi.getter
    def branch(self) -> pulumi.Input[builtins.str]:
        """
        Name of the branch.
        """
        return pulumi.get(self, "branch")

    @branch.setter
    def branch(self, value: pulumi.Input[builtins.str]):
        pulumi.set(self, "branch", value)

    @property
    @pulumi.getter
    def project(self) -> pulumi.Input[builtins.str]:
        """
        The id of the project.
        """
        return pulumi.get(self, "project")

    @project.setter
    def project(self, value: pulumi.Input[builtins.str]):
        pulumi.set(self, "project", value)

    @property
    @pulumi.getter(name="allowForcePush")
    def allow_force_push(self) -> Optional[pulumi.Input[builtins.bool]]:
        """
        Can be set to true to allow users with push access to force push.
        """
        return pulumi.get(self, "allow_force_push")

    @allow_force_push.setter
    def allow_force_push(self, value: Optional[pulumi.Input[builtins.bool]]):
        pulumi.set(self, "allow_force_push", value)

    @property
    @pulumi.getter(name="allowedToMerges")
    def allowed_to_merges(self) -> Optional[pulumi.Input[Sequence[pulumi.Input['BranchProtectionAllowedToMergeArgs']]]]:
        """
        Array of access levels and user(s)/group(s) allowed to merge to protected branch.
        """
        return pulumi.get(self, "allowed_to_merges")

    @allowed_to_merges.setter
    def allowed_to_merges(self, value: Optional[pulumi.Input[Sequence[pulumi.Input['BranchProtectionAllowedToMergeArgs']]]]):
        pulumi.set(self, "allowed_to_merges", value)

    @property
    @pulumi.getter(name="allowedToPushes")
    def allowed_to_pushes(self) -> Optional[pulumi.Input[Sequence[pulumi.Input['BranchProtectionAllowedToPushArgs']]]]:
        """
        Array of access levels and user(s)/group(s) allowed to push to protected branch.
        """
        return pulumi.get(self, "allowed_to_pushes")

    @allowed_to_pushes.setter
    def allowed_to_pushes(self, value: Optional[pulumi.Input[Sequence[pulumi.Input['BranchProtectionAllowedToPushArgs']]]]):
        pulumi.set(self, "allowed_to_pushes", value)

    @property
    @pulumi.getter(name="allowedToUnprotects")
    def allowed_to_unprotects(self) -> Optional[pulumi.Input[Sequence[pulumi.Input['BranchProtectionAllowedToUnprotectArgs']]]]:
        """
        Array of access levels and user(s)/group(s) allowed to unprotect push to protected branch.
        """
        return pulumi.get(self, "allowed_to_unprotects")

    @allowed_to_unprotects.setter
    def allowed_to_unprotects(self, value: Optional[pulumi.Input[Sequence[pulumi.Input['BranchProtectionAllowedToUnprotectArgs']]]]):
        pulumi.set(self, "allowed_to_unprotects", value)

    @property
    @pulumi.getter(name="codeOwnerApprovalRequired")
    def code_owner_approval_required(self) -> Optional[pulumi.Input[builtins.bool]]:
        """
        Can be set to true to require code owner approval before merging. Only available for Premium and Ultimate instances.
        """
        return pulumi.get(self, "code_owner_approval_required")

    @code_owner_approval_required.setter
    def code_owner_approval_required(self, value: Optional[pulumi.Input[builtins.bool]]):
        pulumi.set(self, "code_owner_approval_required", value)

    @property
    @pulumi.getter(name="mergeAccessLevel")
    def merge_access_level(self) -> Optional[pulumi.Input[builtins.str]]:
        """
        Access levels allowed to merge. Valid values are: `no one`, `developer`, `maintainer`.
        """
        return pulumi.get(self, "merge_access_level")

    @merge_access_level.setter
    def merge_access_level(self, value: Optional[pulumi.Input[builtins.str]]):
        pulumi.set(self, "merge_access_level", value)

    @property
    @pulumi.getter(name="pushAccessLevel")
    def push_access_level(self) -> Optional[pulumi.Input[builtins.str]]:
        """
        Access levels allowed to push. Valid values are: `no one`, `developer`, `maintainer`.
        """
        return pulumi.get(self, "push_access_level")

    @push_access_level.setter
    def push_access_level(self, value: Optional[pulumi.Input[builtins.str]]):
        pulumi.set(self, "push_access_level", value)

    @property
    @pulumi.getter(name="unprotectAccessLevel")
    def unprotect_access_level(self) -> Optional[pulumi.Input[builtins.str]]:
        """
        Access levels allowed to unprotect. Valid values are: `developer`, `maintainer`, `admin`.
        """
        return pulumi.get(self, "unprotect_access_level")

    @unprotect_access_level.setter
    def unprotect_access_level(self, value: Optional[pulumi.Input[builtins.str]]):
        pulumi.set(self, "unprotect_access_level", value)


@pulumi.input_type
class _BranchProtectionState:
    def __init__(__self__, *,
                 allow_force_push: Optional[pulumi.Input[builtins.bool]] = None,
                 allowed_to_merges: Optional[pulumi.Input[Sequence[pulumi.Input['BranchProtectionAllowedToMergeArgs']]]] = None,
                 allowed_to_pushes: Optional[pulumi.Input[Sequence[pulumi.Input['BranchProtectionAllowedToPushArgs']]]] = None,
                 allowed_to_unprotects: Optional[pulumi.Input[Sequence[pulumi.Input['BranchProtectionAllowedToUnprotectArgs']]]] = None,
                 branch: Optional[pulumi.Input[builtins.str]] = None,
                 branch_protection_id: Optional[pulumi.Input[builtins.int]] = None,
                 code_owner_approval_required: Optional[pulumi.Input[builtins.bool]] = None,
                 merge_access_level: Optional[pulumi.Input[builtins.str]] = None,
                 project: Optional[pulumi.Input[builtins.str]] = None,
                 push_access_level: Optional[pulumi.Input[builtins.str]] = None,
                 unprotect_access_level: Optional[pulumi.Input[builtins.str]] = None):
        """
        Input properties used for looking up and filtering BranchProtection resources.
        :param pulumi.Input[builtins.bool] allow_force_push: Can be set to true to allow users with push access to force push.
        :param pulumi.Input[Sequence[pulumi.Input['BranchProtectionAllowedToMergeArgs']]] allowed_to_merges: Array of access levels and user(s)/group(s) allowed to merge to protected branch.
        :param pulumi.Input[Sequence[pulumi.Input['BranchProtectionAllowedToPushArgs']]] allowed_to_pushes: Array of access levels and user(s)/group(s) allowed to push to protected branch.
        :param pulumi.Input[Sequence[pulumi.Input['BranchProtectionAllowedToUnprotectArgs']]] allowed_to_unprotects: Array of access levels and user(s)/group(s) allowed to unprotect push to protected branch.
        :param pulumi.Input[builtins.str] branch: Name of the branch.
        :param pulumi.Input[builtins.int] branch_protection_id: The ID of the branch protection (not the branch name).
        :param pulumi.Input[builtins.bool] code_owner_approval_required: Can be set to true to require code owner approval before merging. Only available for Premium and Ultimate instances.
        :param pulumi.Input[builtins.str] merge_access_level: Access levels allowed to merge. Valid values are: `no one`, `developer`, `maintainer`.
        :param pulumi.Input[builtins.str] project: The id of the project.
        :param pulumi.Input[builtins.str] push_access_level: Access levels allowed to push. Valid values are: `no one`, `developer`, `maintainer`.
        :param pulumi.Input[builtins.str] unprotect_access_level: Access levels allowed to unprotect. Valid values are: `developer`, `maintainer`, `admin`.
        """
        if allow_force_push is not None:
            pulumi.set(__self__, "allow_force_push", allow_force_push)
        if allowed_to_merges is not None:
            pulumi.set(__self__, "allowed_to_merges", allowed_to_merges)
        if allowed_to_pushes is not None:
            pulumi.set(__self__, "allowed_to_pushes", allowed_to_pushes)
        if allowed_to_unprotects is not None:
            pulumi.set(__self__, "allowed_to_unprotects", allowed_to_unprotects)
        if branch is not None:
            pulumi.set(__self__, "branch", branch)
        if branch_protection_id is not None:
            pulumi.set(__self__, "branch_protection_id", branch_protection_id)
        if code_owner_approval_required is not None:
            pulumi.set(__self__, "code_owner_approval_required", code_owner_approval_required)
        if merge_access_level is not None:
            pulumi.set(__self__, "merge_access_level", merge_access_level)
        if project is not None:
            pulumi.set(__self__, "project", project)
        if push_access_level is not None:
            pulumi.set(__self__, "push_access_level", push_access_level)
        if unprotect_access_level is not None:
            pulumi.set(__self__, "unprotect_access_level", unprotect_access_level)

    @property
    @pulumi.getter(name="allowForcePush")
    def allow_force_push(self) -> Optional[pulumi.Input[builtins.bool]]:
        """
        Can be set to true to allow users with push access to force push.
        """
        return pulumi.get(self, "allow_force_push")

    @allow_force_push.setter
    def allow_force_push(self, value: Optional[pulumi.Input[builtins.bool]]):
        pulumi.set(self, "allow_force_push", value)

    @property
    @pulumi.getter(name="allowedToMerges")
    def allowed_to_merges(self) -> Optional[pulumi.Input[Sequence[pulumi.Input['BranchProtectionAllowedToMergeArgs']]]]:
        """
        Array of access levels and user(s)/group(s) allowed to merge to protected branch.
        """
        return pulumi.get(self, "allowed_to_merges")

    @allowed_to_merges.setter
    def allowed_to_merges(self, value: Optional[pulumi.Input[Sequence[pulumi.Input['BranchProtectionAllowedToMergeArgs']]]]):
        pulumi.set(self, "allowed_to_merges", value)

    @property
    @pulumi.getter(name="allowedToPushes")
    def allowed_to_pushes(self) -> Optional[pulumi.Input[Sequence[pulumi.Input['BranchProtectionAllowedToPushArgs']]]]:
        """
        Array of access levels and user(s)/group(s) allowed to push to protected branch.
        """
        return pulumi.get(self, "allowed_to_pushes")

    @allowed_to_pushes.setter
    def allowed_to_pushes(self, value: Optional[pulumi.Input[Sequence[pulumi.Input['BranchProtectionAllowedToPushArgs']]]]):
        pulumi.set(self, "allowed_to_pushes", value)

    @property
    @pulumi.getter(name="allowedToUnprotects")
    def allowed_to_unprotects(self) -> Optional[pulumi.Input[Sequence[pulumi.Input['BranchProtectionAllowedToUnprotectArgs']]]]:
        """
        Array of access levels and user(s)/group(s) allowed to unprotect push to protected branch.
        """
        return pulumi.get(self, "allowed_to_unprotects")

    @allowed_to_unprotects.setter
    def allowed_to_unprotects(self, value: Optional[pulumi.Input[Sequence[pulumi.Input['BranchProtectionAllowedToUnprotectArgs']]]]):
        pulumi.set(self, "allowed_to_unprotects", value)

    @property
    @pulumi.getter
    def branch(self) -> Optional[pulumi.Input[builtins.str]]:
        """
        Name of the branch.
        """
        return pulumi.get(self, "branch")

    @branch.setter
    def branch(self, value: Optional[pulumi.Input[builtins.str]]):
        pulumi.set(self, "branch", value)

    @property
    @pulumi.getter(name="branchProtectionId")
    def branch_protection_id(self) -> Optional[pulumi.Input[builtins.int]]:
        """
        The ID of the branch protection (not the branch name).
        """
        return pulumi.get(self, "branch_protection_id")

    @branch_protection_id.setter
    def branch_protection_id(self, value: Optional[pulumi.Input[builtins.int]]):
        pulumi.set(self, "branch_protection_id", value)

    @property
    @pulumi.getter(name="codeOwnerApprovalRequired")
    def code_owner_approval_required(self) -> Optional[pulumi.Input[builtins.bool]]:
        """
        Can be set to true to require code owner approval before merging. Only available for Premium and Ultimate instances.
        """
        return pulumi.get(self, "code_owner_approval_required")

    @code_owner_approval_required.setter
    def code_owner_approval_required(self, value: Optional[pulumi.Input[builtins.bool]]):
        pulumi.set(self, "code_owner_approval_required", value)

    @property
    @pulumi.getter(name="mergeAccessLevel")
    def merge_access_level(self) -> Optional[pulumi.Input[builtins.str]]:
        """
        Access levels allowed to merge. Valid values are: `no one`, `developer`, `maintainer`.
        """
        return pulumi.get(self, "merge_access_level")

    @merge_access_level.setter
    def merge_access_level(self, value: Optional[pulumi.Input[builtins.str]]):
        pulumi.set(self, "merge_access_level", value)

    @property
    @pulumi.getter
    def project(self) -> Optional[pulumi.Input[builtins.str]]:
        """
        The id of the project.
        """
        return pulumi.get(self, "project")

    @project.setter
    def project(self, value: Optional[pulumi.Input[builtins.str]]):
        pulumi.set(self, "project", value)

    @property
    @pulumi.getter(name="pushAccessLevel")
    def push_access_level(self) -> Optional[pulumi.Input[builtins.str]]:
        """
        Access levels allowed to push. Valid values are: `no one`, `developer`, `maintainer`.
        """
        return pulumi.get(self, "push_access_level")

    @push_access_level.setter
    def push_access_level(self, value: Optional[pulumi.Input[builtins.str]]):
        pulumi.set(self, "push_access_level", value)

    @property
    @pulumi.getter(name="unprotectAccessLevel")
    def unprotect_access_level(self) -> Optional[pulumi.Input[builtins.str]]:
        """
        Access levels allowed to unprotect. Valid values are: `developer`, `maintainer`, `admin`.
        """
        return pulumi.get(self, "unprotect_access_level")

    @unprotect_access_level.setter
    def unprotect_access_level(self, value: Optional[pulumi.Input[builtins.str]]):
        pulumi.set(self, "unprotect_access_level", value)


class BranchProtection(pulumi.CustomResource):

    pulumi_type = "gitlab:index/branchProtection:BranchProtection"

    @overload
    def __init__(__self__,
                 resource_name: str,
                 opts: Optional[pulumi.ResourceOptions] = None,
                 allow_force_push: Optional[pulumi.Input[builtins.bool]] = None,
                 allowed_to_merges: Optional[pulumi.Input[Sequence[pulumi.Input[Union['BranchProtectionAllowedToMergeArgs', 'BranchProtectionAllowedToMergeArgsDict']]]]] = None,
                 allowed_to_pushes: Optional[pulumi.Input[Sequence[pulumi.Input[Union['BranchProtectionAllowedToPushArgs', 'BranchProtectionAllowedToPushArgsDict']]]]] = None,
                 allowed_to_unprotects: Optional[pulumi.Input[Sequence[pulumi.Input[Union['BranchProtectionAllowedToUnprotectArgs', 'BranchProtectionAllowedToUnprotectArgsDict']]]]] = None,
                 branch: Optional[pulumi.Input[builtins.str]] = None,
                 code_owner_approval_required: Optional[pulumi.Input[builtins.bool]] = None,
                 merge_access_level: Optional[pulumi.Input[builtins.str]] = None,
                 project: Optional[pulumi.Input[builtins.str]] = None,
                 push_access_level: Optional[pulumi.Input[builtins.str]] = None,
                 unprotect_access_level: Optional[pulumi.Input[builtins.str]] = None,
                 __props__=None):
        """
        ## Example Usage

        ```python
        import pulumi
        import pulumi_gitlab as gitlab

        branch_protect = gitlab.BranchProtection("BranchProtect",
            project="12345",
            branch="BranchProtected",
            push_access_level="developer",
            merge_access_level="developer",
            unprotect_access_level="developer",
            allow_force_push=True,
            code_owner_approval_required=True,
            allowed_to_pushes=[
                {
                    "user_id": 5,
                },
                {
                    "user_id": 521,
                },
            ],
            allowed_to_merges=[
                {
                    "user_id": 15,
                },
                {
                    "user_id": 37,
                },
            ],
            allowed_to_unprotects=[
                {
                    "user_id": 15,
                },
                {
                    "group_id": 42,
                },
            ])
        # Example using dynamic block
        main = gitlab.BranchProtection("main",
            allowed_to_pushes=[{
                "user_id": entry["value"],
            } for entry in [{"key": k, "value": v} for k, v in [
                50,
                55,
                60,
            ]]],
            project="12345",
            branch="main",
            push_access_level="maintainer",
            merge_access_level="maintainer",
            unprotect_access_level="maintainer")
        ```

        ## Import

        Starting in Terraform v1.5.0 you can use an import block to import `gitlab_branch_protection`. For example:

        terraform

        import {

          to = gitlab_branch_protection.example

          id = "see CLI command below for ID"

        }

        Import using the CLI is supported using the following syntax:

        Gitlab protected branches can be imported with a key composed of `<project_id>:<branch>`, e.g.

        ```sh
        $ pulumi import gitlab:index/branchProtection:BranchProtection BranchProtect "12345:main"
        ```

        :param str resource_name: The name of the resource.
        :param pulumi.ResourceOptions opts: Options for the resource.
        :param pulumi.Input[builtins.bool] allow_force_push: Can be set to true to allow users with push access to force push.
        :param pulumi.Input[Sequence[pulumi.Input[Union['BranchProtectionAllowedToMergeArgs', 'BranchProtectionAllowedToMergeArgsDict']]]] allowed_to_merges: Array of access levels and user(s)/group(s) allowed to merge to protected branch.
        :param pulumi.Input[Sequence[pulumi.Input[Union['BranchProtectionAllowedToPushArgs', 'BranchProtectionAllowedToPushArgsDict']]]] allowed_to_pushes: Array of access levels and user(s)/group(s) allowed to push to protected branch.
        :param pulumi.Input[Sequence[pulumi.Input[Union['BranchProtectionAllowedToUnprotectArgs', 'BranchProtectionAllowedToUnprotectArgsDict']]]] allowed_to_unprotects: Array of access levels and user(s)/group(s) allowed to unprotect push to protected branch.
        :param pulumi.Input[builtins.str] branch: Name of the branch.
        :param pulumi.Input[builtins.bool] code_owner_approval_required: Can be set to true to require code owner approval before merging. Only available for Premium and Ultimate instances.
        :param pulumi.Input[builtins.str] merge_access_level: Access levels allowed to merge. Valid values are: `no one`, `developer`, `maintainer`.
        :param pulumi.Input[builtins.str] project: The id of the project.
        :param pulumi.Input[builtins.str] push_access_level: Access levels allowed to push. Valid values are: `no one`, `developer`, `maintainer`.
        :param pulumi.Input[builtins.str] unprotect_access_level: Access levels allowed to unprotect. Valid values are: `developer`, `maintainer`, `admin`.
        """
        ...
    @overload
    def __init__(__self__,
                 resource_name: str,
                 args: BranchProtectionArgs,
                 opts: Optional[pulumi.ResourceOptions] = None):
        """
        ## Example Usage

        ```python
        import pulumi
        import pulumi_gitlab as gitlab

        branch_protect = gitlab.BranchProtection("BranchProtect",
            project="12345",
            branch="BranchProtected",
            push_access_level="developer",
            merge_access_level="developer",
            unprotect_access_level="developer",
            allow_force_push=True,
            code_owner_approval_required=True,
            allowed_to_pushes=[
                {
                    "user_id": 5,
                },
                {
                    "user_id": 521,
                },
            ],
            allowed_to_merges=[
                {
                    "user_id": 15,
                },
                {
                    "user_id": 37,
                },
            ],
            allowed_to_unprotects=[
                {
                    "user_id": 15,
                },
                {
                    "group_id": 42,
                },
            ])
        # Example using dynamic block
        main = gitlab.BranchProtection("main",
            allowed_to_pushes=[{
                "user_id": entry["value"],
            } for entry in [{"key": k, "value": v} for k, v in [
                50,
                55,
                60,
            ]]],
            project="12345",
            branch="main",
            push_access_level="maintainer",
            merge_access_level="maintainer",
            unprotect_access_level="maintainer")
        ```

        ## Import

        Starting in Terraform v1.5.0 you can use an import block to import `gitlab_branch_protection`. For example:

        terraform

        import {

          to = gitlab_branch_protection.example

          id = "see CLI command below for ID"

        }

        Import using the CLI is supported using the following syntax:

        Gitlab protected branches can be imported with a key composed of `<project_id>:<branch>`, e.g.

        ```sh
        $ pulumi import gitlab:index/branchProtection:BranchProtection BranchProtect "12345:main"
        ```

        :param str resource_name: The name of the resource.
        :param BranchProtectionArgs args: The arguments to use to populate this resource's properties.
        :param pulumi.ResourceOptions opts: Options for the resource.
        """
        ...
    def __init__(__self__, resource_name: str, *args, **kwargs):
        resource_args, opts = _utilities.get_resource_args_opts(BranchProtectionArgs, pulumi.ResourceOptions, *args, **kwargs)
        if resource_args is not None:
            __self__._internal_init(resource_name, opts, **resource_args.__dict__)
        else:
            __self__._internal_init(resource_name, *args, **kwargs)

    def _internal_init(__self__,
                 resource_name: str,
                 opts: Optional[pulumi.ResourceOptions] = None,
                 allow_force_push: Optional[pulumi.Input[builtins.bool]] = None,
                 allowed_to_merges: Optional[pulumi.Input[Sequence[pulumi.Input[Union['BranchProtectionAllowedToMergeArgs', 'BranchProtectionAllowedToMergeArgsDict']]]]] = None,
                 allowed_to_pushes: Optional[pulumi.Input[Sequence[pulumi.Input[Union['BranchProtectionAllowedToPushArgs', 'BranchProtectionAllowedToPushArgsDict']]]]] = None,
                 allowed_to_unprotects: Optional[pulumi.Input[Sequence[pulumi.Input[Union['BranchProtectionAllowedToUnprotectArgs', 'BranchProtectionAllowedToUnprotectArgsDict']]]]] = None,
                 branch: Optional[pulumi.Input[builtins.str]] = None,
                 code_owner_approval_required: Optional[pulumi.Input[builtins.bool]] = None,
                 merge_access_level: Optional[pulumi.Input[builtins.str]] = None,
                 project: Optional[pulumi.Input[builtins.str]] = None,
                 push_access_level: Optional[pulumi.Input[builtins.str]] = None,
                 unprotect_access_level: Optional[pulumi.Input[builtins.str]] = None,
                 __props__=None):
        opts = pulumi.ResourceOptions.merge(_utilities.get_resource_opts_defaults(), opts)
        if not isinstance(opts, pulumi.ResourceOptions):
            raise TypeError('Expected resource options to be a ResourceOptions instance')
        if opts.id is None:
            if __props__ is not None:
                raise TypeError('__props__ is only valid when passed in combination with a valid opts.id to get an existing resource')
            __props__ = BranchProtectionArgs.__new__(BranchProtectionArgs)

            __props__.__dict__["allow_force_push"] = allow_force_push
            __props__.__dict__["allowed_to_merges"] = allowed_to_merges
            __props__.__dict__["allowed_to_pushes"] = allowed_to_pushes
            __props__.__dict__["allowed_to_unprotects"] = allowed_to_unprotects
            if branch is None and not opts.urn:
                raise TypeError("Missing required property 'branch'")
            __props__.__dict__["branch"] = branch
            __props__.__dict__["code_owner_approval_required"] = code_owner_approval_required
            __props__.__dict__["merge_access_level"] = merge_access_level
            if project is None and not opts.urn:
                raise TypeError("Missing required property 'project'")
            __props__.__dict__["project"] = project
            __props__.__dict__["push_access_level"] = push_access_level
            __props__.__dict__["unprotect_access_level"] = unprotect_access_level
            __props__.__dict__["branch_protection_id"] = None
        super(BranchProtection, __self__).__init__(
            'gitlab:index/branchProtection:BranchProtection',
            resource_name,
            __props__,
            opts)

    @staticmethod
    def get(resource_name: str,
            id: pulumi.Input[str],
            opts: Optional[pulumi.ResourceOptions] = None,
            allow_force_push: Optional[pulumi.Input[builtins.bool]] = None,
            allowed_to_merges: Optional[pulumi.Input[Sequence[pulumi.Input[Union['BranchProtectionAllowedToMergeArgs', 'BranchProtectionAllowedToMergeArgsDict']]]]] = None,
            allowed_to_pushes: Optional[pulumi.Input[Sequence[pulumi.Input[Union['BranchProtectionAllowedToPushArgs', 'BranchProtectionAllowedToPushArgsDict']]]]] = None,
            allowed_to_unprotects: Optional[pulumi.Input[Sequence[pulumi.Input[Union['BranchProtectionAllowedToUnprotectArgs', 'BranchProtectionAllowedToUnprotectArgsDict']]]]] = None,
            branch: Optional[pulumi.Input[builtins.str]] = None,
            branch_protection_id: Optional[pulumi.Input[builtins.int]] = None,
            code_owner_approval_required: Optional[pulumi.Input[builtins.bool]] = None,
            merge_access_level: Optional[pulumi.Input[builtins.str]] = None,
            project: Optional[pulumi.Input[builtins.str]] = None,
            push_access_level: Optional[pulumi.Input[builtins.str]] = None,
            unprotect_access_level: Optional[pulumi.Input[builtins.str]] = None) -> 'BranchProtection':
        """
        Get an existing BranchProtection resource's state with the given name, id, and optional extra
        properties used to qualify the lookup.

        :param str resource_name: The unique name of the resulting resource.
        :param pulumi.Input[str] id: The unique provider ID of the resource to lookup.
        :param pulumi.ResourceOptions opts: Options for the resource.
        :param pulumi.Input[builtins.bool] allow_force_push: Can be set to true to allow users with push access to force push.
        :param pulumi.Input[Sequence[pulumi.Input[Union['BranchProtectionAllowedToMergeArgs', 'BranchProtectionAllowedToMergeArgsDict']]]] allowed_to_merges: Array of access levels and user(s)/group(s) allowed to merge to protected branch.
        :param pulumi.Input[Sequence[pulumi.Input[Union['BranchProtectionAllowedToPushArgs', 'BranchProtectionAllowedToPushArgsDict']]]] allowed_to_pushes: Array of access levels and user(s)/group(s) allowed to push to protected branch.
        :param pulumi.Input[Sequence[pulumi.Input[Union['BranchProtectionAllowedToUnprotectArgs', 'BranchProtectionAllowedToUnprotectArgsDict']]]] allowed_to_unprotects: Array of access levels and user(s)/group(s) allowed to unprotect push to protected branch.
        :param pulumi.Input[builtins.str] branch: Name of the branch.
        :param pulumi.Input[builtins.int] branch_protection_id: The ID of the branch protection (not the branch name).
        :param pulumi.Input[builtins.bool] code_owner_approval_required: Can be set to true to require code owner approval before merging. Only available for Premium and Ultimate instances.
        :param pulumi.Input[builtins.str] merge_access_level: Access levels allowed to merge. Valid values are: `no one`, `developer`, `maintainer`.
        :param pulumi.Input[builtins.str] project: The id of the project.
        :param pulumi.Input[builtins.str] push_access_level: Access levels allowed to push. Valid values are: `no one`, `developer`, `maintainer`.
        :param pulumi.Input[builtins.str] unprotect_access_level: Access levels allowed to unprotect. Valid values are: `developer`, `maintainer`, `admin`.
        """
        opts = pulumi.ResourceOptions.merge(opts, pulumi.ResourceOptions(id=id))

        __props__ = _BranchProtectionState.__new__(_BranchProtectionState)

        __props__.__dict__["allow_force_push"] = allow_force_push
        __props__.__dict__["allowed_to_merges"] = allowed_to_merges
        __props__.__dict__["allowed_to_pushes"] = allowed_to_pushes
        __props__.__dict__["allowed_to_unprotects"] = allowed_to_unprotects
        __props__.__dict__["branch"] = branch
        __props__.__dict__["branch_protection_id"] = branch_protection_id
        __props__.__dict__["code_owner_approval_required"] = code_owner_approval_required
        __props__.__dict__["merge_access_level"] = merge_access_level
        __props__.__dict__["project"] = project
        __props__.__dict__["push_access_level"] = push_access_level
        __props__.__dict__["unprotect_access_level"] = unprotect_access_level
        return BranchProtection(resource_name, opts=opts, __props__=__props__)

    @property
    @pulumi.getter(name="allowForcePush")
    def allow_force_push(self) -> pulumi.Output[builtins.bool]:
        """
        Can be set to true to allow users with push access to force push.
        """
        return pulumi.get(self, "allow_force_push")

    @property
    @pulumi.getter(name="allowedToMerges")
    def allowed_to_merges(self) -> pulumi.Output[Optional[Sequence['outputs.BranchProtectionAllowedToMerge']]]:
        """
        Array of access levels and user(s)/group(s) allowed to merge to protected branch.
        """
        return pulumi.get(self, "allowed_to_merges")

    @property
    @pulumi.getter(name="allowedToPushes")
    def allowed_to_pushes(self) -> pulumi.Output[Optional[Sequence['outputs.BranchProtectionAllowedToPush']]]:
        """
        Array of access levels and user(s)/group(s) allowed to push to protected branch.
        """
        return pulumi.get(self, "allowed_to_pushes")

    @property
    @pulumi.getter(name="allowedToUnprotects")
    def allowed_to_unprotects(self) -> pulumi.Output[Optional[Sequence['outputs.BranchProtectionAllowedToUnprotect']]]:
        """
        Array of access levels and user(s)/group(s) allowed to unprotect push to protected branch.
        """
        return pulumi.get(self, "allowed_to_unprotects")

    @property
    @pulumi.getter
    def branch(self) -> pulumi.Output[builtins.str]:
        """
        Name of the branch.
        """
        return pulumi.get(self, "branch")

    @property
    @pulumi.getter(name="branchProtectionId")
    def branch_protection_id(self) -> pulumi.Output[builtins.int]:
        """
        The ID of the branch protection (not the branch name).
        """
        return pulumi.get(self, "branch_protection_id")

    @property
    @pulumi.getter(name="codeOwnerApprovalRequired")
    def code_owner_approval_required(self) -> pulumi.Output[builtins.bool]:
        """
        Can be set to true to require code owner approval before merging. Only available for Premium and Ultimate instances.
        """
        return pulumi.get(self, "code_owner_approval_required")

    @property
    @pulumi.getter(name="mergeAccessLevel")
    def merge_access_level(self) -> pulumi.Output[builtins.str]:
        """
        Access levels allowed to merge. Valid values are: `no one`, `developer`, `maintainer`.
        """
        return pulumi.get(self, "merge_access_level")

    @property
    @pulumi.getter
    def project(self) -> pulumi.Output[builtins.str]:
        """
        The id of the project.
        """
        return pulumi.get(self, "project")

    @property
    @pulumi.getter(name="pushAccessLevel")
    def push_access_level(self) -> pulumi.Output[builtins.str]:
        """
        Access levels allowed to push. Valid values are: `no one`, `developer`, `maintainer`.
        """
        return pulumi.get(self, "push_access_level")

    @property
    @pulumi.getter(name="unprotectAccessLevel")
    def unprotect_access_level(self) -> pulumi.Output[builtins.str]:
        """
        Access levels allowed to unprotect. Valid values are: `developer`, `maintainer`, `admin`.
        """
        return pulumi.get(self, "unprotect_access_level")




# coding=utf-8
# *** WARNING: this file was generated by the Pulumi Terraform Bridge (tfgen) Tool. ***
# *** Do not edit by hand unless you're certain you know what you are doing! ***

import builtins
import copy
import warnings
import sys
import pulumi
import pulumi.runtime
from typing import Any, Mapping, Optional, Sequence, Union, overload
if sys.version_info >= (3, 11):
    from typing import NotRequired, TypedDict, TypeAlias
else:
    from typing_extensions import NotRequired, TypedDict, TypeAlias
from . import _utilities

__all__ = [
    'GetProjectHookResult',
    'AwaitableGetProjectHookResult',
    'get_project_hook',
    'get_project_hook_output',
]

@pulumi.output_type
class GetProjectHookResult:
    """
    A collection of values returned by getProjectHook.
    """
    def __init__(__self__, confidential_issues_events=None, confidential_note_events=None, custom_webhook_template=None, deployment_events=None, enable_ssl_verification=None, hook_id=None, id=None, issues_events=None, job_events=None, merge_requests_events=None, note_events=None, pipeline_events=None, project=None, project_id=None, push_events=None, push_events_branch_filter=None, releases_events=None, tag_push_events=None, token=None, url=None, wiki_page_events=None):
        if confidential_issues_events and not isinstance(confidential_issues_events, bool):
            raise TypeError("Expected argument 'confidential_issues_events' to be a bool")
        pulumi.set(__self__, "confidential_issues_events", confidential_issues_events)
        if confidential_note_events and not isinstance(confidential_note_events, bool):
            raise TypeError("Expected argument 'confidential_note_events' to be a bool")
        pulumi.set(__self__, "confidential_note_events", confidential_note_events)
        if custom_webhook_template and not isinstance(custom_webhook_template, str):
            raise TypeError("Expected argument 'custom_webhook_template' to be a str")
        pulumi.set(__self__, "custom_webhook_template", custom_webhook_template)
        if deployment_events and not isinstance(deployment_events, bool):
            raise TypeError("Expected argument 'deployment_events' to be a bool")
        pulumi.set(__self__, "deployment_events", deployment_events)
        if enable_ssl_verification and not isinstance(enable_ssl_verification, bool):
            raise TypeError("Expected argument 'enable_ssl_verification' to be a bool")
        pulumi.set(__self__, "enable_ssl_verification", enable_ssl_verification)
        if hook_id and not isinstance(hook_id, int):
            raise TypeError("Expected argument 'hook_id' to be a int")
        pulumi.set(__self__, "hook_id", hook_id)
        if id and not isinstance(id, str):
            raise TypeError("Expected argument 'id' to be a str")
        pulumi.set(__self__, "id", id)
        if issues_events and not isinstance(issues_events, bool):
            raise TypeError("Expected argument 'issues_events' to be a bool")
        pulumi.set(__self__, "issues_events", issues_events)
        if job_events and not isinstance(job_events, bool):
            raise TypeError("Expected argument 'job_events' to be a bool")
        pulumi.set(__self__, "job_events", job_events)
        if merge_requests_events and not isinstance(merge_requests_events, bool):
            raise TypeError("Expected argument 'merge_requests_events' to be a bool")
        pulumi.set(__self__, "merge_requests_events", merge_requests_events)
        if note_events and not isinstance(note_events, bool):
            raise TypeError("Expected argument 'note_events' to be a bool")
        pulumi.set(__self__, "note_events", note_events)
        if pipeline_events and not isinstance(pipeline_events, bool):
            raise TypeError("Expected argument 'pipeline_events' to be a bool")
        pulumi.set(__self__, "pipeline_events", pipeline_events)
        if project and not isinstance(project, str):
            raise TypeError("Expected argument 'project' to be a str")
        pulumi.set(__self__, "project", project)
        if project_id and not isinstance(project_id, int):
            raise TypeError("Expected argument 'project_id' to be a int")
        pulumi.set(__self__, "project_id", project_id)
        if push_events and not isinstance(push_events, bool):
            raise TypeError("Expected argument 'push_events' to be a bool")
        pulumi.set(__self__, "push_events", push_events)
        if push_events_branch_filter and not isinstance(push_events_branch_filter, str):
            raise TypeError("Expected argument 'push_events_branch_filter' to be a str")
        pulumi.set(__self__, "push_events_branch_filter", push_events_branch_filter)
        if releases_events and not isinstance(releases_events, bool):
            raise TypeError("Expected argument 'releases_events' to be a bool")
        pulumi.set(__self__, "releases_events", releases_events)
        if tag_push_events and not isinstance(tag_push_events, bool):
            raise TypeError("Expected argument 'tag_push_events' to be a bool")
        pulumi.set(__self__, "tag_push_events", tag_push_events)
        if token and not isinstance(token, str):
            raise TypeError("Expected argument 'token' to be a str")
        pulumi.set(__self__, "token", token)
        if url and not isinstance(url, str):
            raise TypeError("Expected argument 'url' to be a str")
        pulumi.set(__self__, "url", url)
        if wiki_page_events and not isinstance(wiki_page_events, bool):
            raise TypeError("Expected argument 'wiki_page_events' to be a bool")
        pulumi.set(__self__, "wiki_page_events", wiki_page_events)

    @property
    @pulumi.getter(name="confidentialIssuesEvents")
    def confidential_issues_events(self) -> builtins.bool:
        """
        Invoke the hook for confidential issues events.
        """
        return pulumi.get(self, "confidential_issues_events")

    @property
    @pulumi.getter(name="confidentialNoteEvents")
    def confidential_note_events(self) -> builtins.bool:
        """
        Invoke the hook for confidential notes events.
        """
        return pulumi.get(self, "confidential_note_events")

    @property
    @pulumi.getter(name="customWebhookTemplate")
    def custom_webhook_template(self) -> builtins.str:
        """
        Set a custom webhook template.
        """
        return pulumi.get(self, "custom_webhook_template")

    @property
    @pulumi.getter(name="deploymentEvents")
    def deployment_events(self) -> builtins.bool:
        """
        Invoke the hook for deployment events.
        """
        return pulumi.get(self, "deployment_events")

    @property
    @pulumi.getter(name="enableSslVerification")
    def enable_ssl_verification(self) -> builtins.bool:
        """
        Enable ssl verification when invoking the hook.
        """
        return pulumi.get(self, "enable_ssl_verification")

    @property
    @pulumi.getter(name="hookId")
    def hook_id(self) -> builtins.int:
        """
        The id of the project hook.
        """
        return pulumi.get(self, "hook_id")

    @property
    @pulumi.getter
    def id(self) -> builtins.str:
        """
        The provider-assigned unique ID for this managed resource.
        """
        return pulumi.get(self, "id")

    @property
    @pulumi.getter(name="issuesEvents")
    def issues_events(self) -> builtins.bool:
        """
        Invoke the hook for issues events.
        """
        return pulumi.get(self, "issues_events")

    @property
    @pulumi.getter(name="jobEvents")
    def job_events(self) -> builtins.bool:
        """
        Invoke the hook for job events.
        """
        return pulumi.get(self, "job_events")

    @property
    @pulumi.getter(name="mergeRequestsEvents")
    def merge_requests_events(self) -> builtins.bool:
        """
        Invoke the hook for merge requests.
        """
        return pulumi.get(self, "merge_requests_events")

    @property
    @pulumi.getter(name="noteEvents")
    def note_events(self) -> builtins.bool:
        """
        Invoke the hook for notes events.
        """
        return pulumi.get(self, "note_events")

    @property
    @pulumi.getter(name="pipelineEvents")
    def pipeline_events(self) -> builtins.bool:
        """
        Invoke the hook for pipeline events.
        """
        return pulumi.get(self, "pipeline_events")

    @property
    @pulumi.getter
    def project(self) -> builtins.str:
        """
        The name or id of the project to add the hook to.
        """
        return pulumi.get(self, "project")

    @property
    @pulumi.getter(name="projectId")
    def project_id(self) -> builtins.int:
        """
        The id of the project for the hook.
        """
        return pulumi.get(self, "project_id")

    @property
    @pulumi.getter(name="pushEvents")
    def push_events(self) -> builtins.bool:
        """
        Invoke the hook for push events.
        """
        return pulumi.get(self, "push_events")

    @property
    @pulumi.getter(name="pushEventsBranchFilter")
    def push_events_branch_filter(self) -> builtins.str:
        """
        Invoke the hook for push events on matching branches only.
        """
        return pulumi.get(self, "push_events_branch_filter")

    @property
    @pulumi.getter(name="releasesEvents")
    def releases_events(self) -> builtins.bool:
        """
        Invoke the hook for releases events.
        """
        return pulumi.get(self, "releases_events")

    @property
    @pulumi.getter(name="tagPushEvents")
    def tag_push_events(self) -> builtins.bool:
        """
        Invoke the hook for tag push events.
        """
        return pulumi.get(self, "tag_push_events")

    @property
    @pulumi.getter
    def token(self) -> builtins.str:
        """
        A token to present when invoking the hook. The token is not available for imported resources.
        """
        return pulumi.get(self, "token")

    @property
    @pulumi.getter
    def url(self) -> builtins.str:
        """
        The url of the hook to invoke.
        """
        return pulumi.get(self, "url")

    @property
    @pulumi.getter(name="wikiPageEvents")
    def wiki_page_events(self) -> builtins.bool:
        """
        Invoke the hook for wiki page events.
        """
        return pulumi.get(self, "wiki_page_events")


class AwaitableGetProjectHookResult(GetProjectHookResult):
    # pylint: disable=using-constant-test
    def __await__(self):
        if False:
            yield self
        return GetProjectHookResult(
            confidential_issues_events=self.confidential_issues_events,
            confidential_note_events=self.confidential_note_events,
            custom_webhook_template=self.custom_webhook_template,
            deployment_events=self.deployment_events,
            enable_ssl_verification=self.enable_ssl_verification,
            hook_id=self.hook_id,
            id=self.id,
            issues_events=self.issues_events,
            job_events=self.job_events,
            merge_requests_events=self.merge_requests_events,
            note_events=self.note_events,
            pipeline_events=self.pipeline_events,
            project=self.project,
            project_id=self.project_id,
            push_events=self.push_events,
            push_events_branch_filter=self.push_events_branch_filter,
            releases_events=self.releases_events,
            tag_push_events=self.tag_push_events,
            token=self.token,
            url=self.url,
            wiki_page_events=self.wiki_page_events)


def get_project_hook(hook_id: Optional[builtins.int] = None,
                     project: Optional[builtins.str] = None,
                     opts: Optional[pulumi.InvokeOptions] = None) -> AwaitableGetProjectHookResult:
    """
    The `ProjectHook` data source allows to retrieve details about a hook in a project.

    **Upstream API**: [GitLab REST API docs](https://docs.gitlab.com/api/project_webhooks/#get-a-project-webhook)

    ## Example Usage

    ```python
    import pulumi
    import pulumi_gitlab as gitlab

    example = gitlab.get_project(id="foo/bar/baz")
    example_get_project_hook = gitlab.get_project_hook(project=example.id,
        hook_id=1)
    ```


    :param builtins.int hook_id: The id of the project hook.
    :param builtins.str project: The name or id of the project to add the hook to.
    """
    __args__ = dict()
    __args__['hookId'] = hook_id
    __args__['project'] = project
    opts = pulumi.InvokeOptions.merge(_utilities.get_invoke_opts_defaults(), opts)
    __ret__ = pulumi.runtime.invoke('gitlab:index/getProjectHook:getProjectHook', __args__, opts=opts, typ=GetProjectHookResult).value

    return AwaitableGetProjectHookResult(
        confidential_issues_events=pulumi.get(__ret__, 'confidential_issues_events'),
        confidential_note_events=pulumi.get(__ret__, 'confidential_note_events'),
        custom_webhook_template=pulumi.get(__ret__, 'custom_webhook_template'),
        deployment_events=pulumi.get(__ret__, 'deployment_events'),
        enable_ssl_verification=pulumi.get(__ret__, 'enable_ssl_verification'),
        hook_id=pulumi.get(__ret__, 'hook_id'),
        id=pulumi.get(__ret__, 'id'),
        issues_events=pulumi.get(__ret__, 'issues_events'),
        job_events=pulumi.get(__ret__, 'job_events'),
        merge_requests_events=pulumi.get(__ret__, 'merge_requests_events'),
        note_events=pulumi.get(__ret__, 'note_events'),
        pipeline_events=pulumi.get(__ret__, 'pipeline_events'),
        project=pulumi.get(__ret__, 'project'),
        project_id=pulumi.get(__ret__, 'project_id'),
        push_events=pulumi.get(__ret__, 'push_events'),
        push_events_branch_filter=pulumi.get(__ret__, 'push_events_branch_filter'),
        releases_events=pulumi.get(__ret__, 'releases_events'),
        tag_push_events=pulumi.get(__ret__, 'tag_push_events'),
        token=pulumi.get(__ret__, 'token'),
        url=pulumi.get(__ret__, 'url'),
        wiki_page_events=pulumi.get(__ret__, 'wiki_page_events'))
def get_project_hook_output(hook_id: Optional[pulumi.Input[builtins.int]] = None,
                            project: Optional[pulumi.Input[builtins.str]] = None,
                            opts: Optional[Union[pulumi.InvokeOptions, pulumi.InvokeOutputOptions]] = None) -> pulumi.Output[GetProjectHookResult]:
    """
    The `ProjectHook` data source allows to retrieve details about a hook in a project.

    **Upstream API**: [GitLab REST API docs](https://docs.gitlab.com/api/project_webhooks/#get-a-project-webhook)

    ## Example Usage

    ```python
    import pulumi
    import pulumi_gitlab as gitlab

    example = gitlab.get_project(id="foo/bar/baz")
    example_get_project_hook = gitlab.get_project_hook(project=example.id,
        hook_id=1)
    ```


    :param builtins.int hook_id: The id of the project hook.
    :param builtins.str project: The name or id of the project to add the hook to.
    """
    __args__ = dict()
    __args__['hookId'] = hook_id
    __args__['project'] = project
    opts = pulumi.InvokeOutputOptions.merge(_utilities.get_invoke_opts_defaults(), opts)
    __ret__ = pulumi.runtime.invoke_output('gitlab:index/getProjectHook:getProjectHook', __args__, opts=opts, typ=GetProjectHookResult)
    return __ret__.apply(lambda __response__: GetProjectHookResult(
        confidential_issues_events=pulumi.get(__response__, 'confidential_issues_events'),
        confidential_note_events=pulumi.get(__response__, 'confidential_note_events'),
        custom_webhook_template=pulumi.get(__response__, 'custom_webhook_template'),
        deployment_events=pulumi.get(__response__, 'deployment_events'),
        enable_ssl_verification=pulumi.get(__response__, 'enable_ssl_verification'),
        hook_id=pulumi.get(__response__, 'hook_id'),
        id=pulumi.get(__response__, 'id'),
        issues_events=pulumi.get(__response__, 'issues_events'),
        job_events=pulumi.get(__response__, 'job_events'),
        merge_requests_events=pulumi.get(__response__, 'merge_requests_events'),
        note_events=pulumi.get(__response__, 'note_events'),
        pipeline_events=pulumi.get(__response__, 'pipeline_events'),
        project=pulumi.get(__response__, 'project'),
        project_id=pulumi.get(__response__, 'project_id'),
        push_events=pulumi.get(__response__, 'push_events'),
        push_events_branch_filter=pulumi.get(__response__, 'push_events_branch_filter'),
        releases_events=pulumi.get(__response__, 'releases_events'),
        tag_push_events=pulumi.get(__response__, 'tag_push_events'),
        token=pulumi.get(__response__, 'token'),
        url=pulumi.get(__response__, 'url'),
        wiki_page_events=pulumi.get(__response__, 'wiki_page_events')))



# coding=utf-8
# *** WARNING: this file was generated by the Pulumi Terraform Bridge (tfgen) Tool. ***
# *** Do not edit by hand unless you're certain you know what you are doing! ***

import builtins
import copy
import warnings
import sys
import pulumi
import pulumi.runtime
from typing import Any, Mapping, Optional, Sequence, Union, overload
if sys.version_info >= (3, 11):
    from typing import NotRequired, TypedDict, TypeAlias
else:
    from typing_extensions import NotRequired, TypedDict, TypeAlias
from . import _utilities

__all__ = ['ProjectBadgeArgs', 'ProjectBadge']

@pulumi.input_type
class ProjectBadgeArgs:
    def __init__(__self__, *,
                 image_url: pulumi.Input[builtins.str],
                 link_url: pulumi.Input[builtins.str],
                 project: pulumi.Input[builtins.str],
                 name: Optional[pulumi.Input[builtins.str]] = None):
        """
        The set of arguments for constructing a ProjectBadge resource.
        :param pulumi.Input[builtins.str] image_url: The image url which will be presented on project overview.
        :param pulumi.Input[builtins.str] link_url: The url linked with the badge.
        :param pulumi.Input[builtins.str] project: The id of the project to add the badge to.
        :param pulumi.Input[builtins.str] name: The name of the badge.
        """
        pulumi.set(__self__, "image_url", image_url)
        pulumi.set(__self__, "link_url", link_url)
        pulumi.set(__self__, "project", project)
        if name is not None:
            pulumi.set(__self__, "name", name)

    @property
    @pulumi.getter(name="imageUrl")
    def image_url(self) -> pulumi.Input[builtins.str]:
        """
        The image url which will be presented on project overview.
        """
        return pulumi.get(self, "image_url")

    @image_url.setter
    def image_url(self, value: pulumi.Input[builtins.str]):
        pulumi.set(self, "image_url", value)

    @property
    @pulumi.getter(name="linkUrl")
    def link_url(self) -> pulumi.Input[builtins.str]:
        """
        The url linked with the badge.
        """
        return pulumi.get(self, "link_url")

    @link_url.setter
    def link_url(self, value: pulumi.Input[builtins.str]):
        pulumi.set(self, "link_url", value)

    @property
    @pulumi.getter
    def project(self) -> pulumi.Input[builtins.str]:
        """
        The id of the project to add the badge to.
        """
        return pulumi.get(self, "project")

    @project.setter
    def project(self, value: pulumi.Input[builtins.str]):
        pulumi.set(self, "project", value)

    @property
    @pulumi.getter
    def name(self) -> Optional[pulumi.Input[builtins.str]]:
        """
        The name of the badge.
        """
        return pulumi.get(self, "name")

    @name.setter
    def name(self, value: Optional[pulumi.Input[builtins.str]]):
        pulumi.set(self, "name", value)


@pulumi.input_type
class _ProjectBadgeState:
    def __init__(__self__, *,
                 image_url: Optional[pulumi.Input[builtins.str]] = None,
                 link_url: Optional[pulumi.Input[builtins.str]] = None,
                 name: Optional[pulumi.Input[builtins.str]] = None,
                 project: Optional[pulumi.Input[builtins.str]] = None,
                 rendered_image_url: Optional[pulumi.Input[builtins.str]] = None,
                 rendered_link_url: Optional[pulumi.Input[builtins.str]] = None):
        """
        Input properties used for looking up and filtering ProjectBadge resources.
        :param pulumi.Input[builtins.str] image_url: The image url which will be presented on project overview.
        :param pulumi.Input[builtins.str] link_url: The url linked with the badge.
        :param pulumi.Input[builtins.str] name: The name of the badge.
        :param pulumi.Input[builtins.str] project: The id of the project to add the badge to.
        :param pulumi.Input[builtins.str] rendered_image_url: The image_url argument rendered (in case of use of placeholders).
        :param pulumi.Input[builtins.str] rendered_link_url: The link_url argument rendered (in case of use of placeholders).
        """
        if image_url is not None:
            pulumi.set(__self__, "image_url", image_url)
        if link_url is not None:
            pulumi.set(__self__, "link_url", link_url)
        if name is not None:
            pulumi.set(__self__, "name", name)
        if project is not None:
            pulumi.set(__self__, "project", project)
        if rendered_image_url is not None:
            pulumi.set(__self__, "rendered_image_url", rendered_image_url)
        if rendered_link_url is not None:
            pulumi.set(__self__, "rendered_link_url", rendered_link_url)

    @property
    @pulumi.getter(name="imageUrl")
    def image_url(self) -> Optional[pulumi.Input[builtins.str]]:
        """
        The image url which will be presented on project overview.
        """
        return pulumi.get(self, "image_url")

    @image_url.setter
    def image_url(self, value: Optional[pulumi.Input[builtins.str]]):
        pulumi.set(self, "image_url", value)

    @property
    @pulumi.getter(name="linkUrl")
    def link_url(self) -> Optional[pulumi.Input[builtins.str]]:
        """
        The url linked with the badge.
        """
        return pulumi.get(self, "link_url")

    @link_url.setter
    def link_url(self, value: Optional[pulumi.Input[builtins.str]]):
        pulumi.set(self, "link_url", value)

    @property
    @pulumi.getter
    def name(self) -> Optional[pulumi.Input[builtins.str]]:
        """
        The name of the badge.
        """
        return pulumi.get(self, "name")

    @name.setter
    def name(self, value: Optional[pulumi.Input[builtins.str]]):
        pulumi.set(self, "name", value)

    @property
    @pulumi.getter
    def project(self) -> Optional[pulumi.Input[builtins.str]]:
        """
        The id of the project to add the badge to.
        """
        return pulumi.get(self, "project")

    @project.setter
    def project(self, value: Optional[pulumi.Input[builtins.str]]):
        pulumi.set(self, "project", value)

    @property
    @pulumi.getter(name="renderedImageUrl")
    def rendered_image_url(self) -> Optional[pulumi.Input[builtins.str]]:
        """
        The image_url argument rendered (in case of use of placeholders).
        """
        return pulumi.get(self, "rendered_image_url")

    @rendered_image_url.setter
    def rendered_image_url(self, value: Optional[pulumi.Input[builtins.str]]):
        pulumi.set(self, "rendered_image_url", value)

    @property
    @pulumi.getter(name="renderedLinkUrl")
    def rendered_link_url(self) -> Optional[pulumi.Input[builtins.str]]:
        """
        The link_url argument rendered (in case of use of placeholders).
        """
        return pulumi.get(self, "rendered_link_url")

    @rendered_link_url.setter
    def rendered_link_url(self, value: Optional[pulumi.Input[builtins.str]]):
        pulumi.set(self, "rendered_link_url", value)


class ProjectBadge(pulumi.CustomResource):

    pulumi_type = "gitlab:index/projectBadge:ProjectBadge"

    @overload
    def __init__(__self__,
                 resource_name: str,
                 opts: Optional[pulumi.ResourceOptions] = None,
                 image_url: Optional[pulumi.Input[builtins.str]] = None,
                 link_url: Optional[pulumi.Input[builtins.str]] = None,
                 name: Optional[pulumi.Input[builtins.str]] = None,
                 project: Optional[pulumi.Input[builtins.str]] = None,
                 __props__=None):
        """
        The `ProjectBadge` resource allows to manage the lifecycle of project badges.

        **Upstream API**: [GitLab REST API docs](https://docs.gitlab.com/user/project/badges/#project-badges)

        ## Example Usage

        ```python
        import pulumi
        import pulumi_gitlab as gitlab

        foo = gitlab.Project("foo", name="foo-project")
        example = gitlab.ProjectBadge("example",
            project=foo.id,
            link_url="https://example.com/badge-123",
            image_url="https://example.com/badge-123.svg",
            name="badge-123")
        # Pipeline status badges with placeholders will be enabled
        gitlab_pipeline = gitlab.ProjectBadge("gitlab_pipeline",
            project=foo.id,
            link_url="https://gitlab.example.com/%{project_path}/-/pipelines?ref=%{default_branch}",
            image_url="https://gitlab.example.com/%{project_path}/badges/%{default_branch}/pipeline.svg",
            name="badge-pipeline")
        # Test coverage report badges with placeholders will be enabled
        gitlab_coverage = gitlab.ProjectBadge("gitlab_coverage",
            project=foo.id,
            link_url="https://gitlab.example.com/%{project_path}/-/jobs",
            image_url="https://gitlab.example.com/%{project_path}/badges/%{default_branch}/coverage.svg",
            name="badge-coverage")
        # Latest release badges with placeholders will be enabled
        gitlab_release = gitlab.ProjectBadge("gitlab_release",
            project=foo.id,
            link_url="https://gitlab.example.com/%{project_path}/-/releases",
            image_url="https://gitlab.example.com/%{project_path}/-/badges/release.svg",
            name="badge-release")
        ```

        ## Import

        Starting in Terraform v1.5.0 you can use an import block to import `gitlab_project_badge`. For example:

        terraform

        import {

          to = gitlab_project_badge.example

          id = "see CLI command below for ID"

        }

        Import using the CLI is supported using the following syntax:

        GitLab project badges can be imported using an id made up of `{project_id}:{badge_id}`, e.g.

        ```sh
        $ pulumi import gitlab:index/projectBadge:ProjectBadge foo 1:3
        ```

        :param str resource_name: The name of the resource.
        :param pulumi.ResourceOptions opts: Options for the resource.
        :param pulumi.Input[builtins.str] image_url: The image url which will be presented on project overview.
        :param pulumi.Input[builtins.str] link_url: The url linked with the badge.
        :param pulumi.Input[builtins.str] name: The name of the badge.
        :param pulumi.Input[builtins.str] project: The id of the project to add the badge to.
        """
        ...
    @overload
    def __init__(__self__,
                 resource_name: str,
                 args: ProjectBadgeArgs,
                 opts: Optional[pulumi.ResourceOptions] = None):
        """
        The `ProjectBadge` resource allows to manage the lifecycle of project badges.

        **Upstream API**: [GitLab REST API docs](https://docs.gitlab.com/user/project/badges/#project-badges)

        ## Example Usage

        ```python
        import pulumi
        import pulumi_gitlab as gitlab

        foo = gitlab.Project("foo", name="foo-project")
        example = gitlab.ProjectBadge("example",
            project=foo.id,
            link_url="https://example.com/badge-123",
            image_url="https://example.com/badge-123.svg",
            name="badge-123")
        # Pipeline status badges with placeholders will be enabled
        gitlab_pipeline = gitlab.ProjectBadge("gitlab_pipeline",
            project=foo.id,
            link_url="https://gitlab.example.com/%{project_path}/-/pipelines?ref=%{default_branch}",
            image_url="https://gitlab.example.com/%{project_path}/badges/%{default_branch}/pipeline.svg",
            name="badge-pipeline")
        # Test coverage report badges with placeholders will be enabled
        gitlab_coverage = gitlab.ProjectBadge("gitlab_coverage",
            project=foo.id,
            link_url="https://gitlab.example.com/%{project_path}/-/jobs",
            image_url="https://gitlab.example.com/%{project_path}/badges/%{default_branch}/coverage.svg",
            name="badge-coverage")
        # Latest release badges with placeholders will be enabled
        gitlab_release = gitlab.ProjectBadge("gitlab_release",
            project=foo.id,
            link_url="https://gitlab.example.com/%{project_path}/-/releases",
            image_url="https://gitlab.example.com/%{project_path}/-/badges/release.svg",
            name="badge-release")
        ```

        ## Import

        Starting in Terraform v1.5.0 you can use an import block to import `gitlab_project_badge`. For example:

        terraform

        import {

          to = gitlab_project_badge.example

          id = "see CLI command below for ID"

        }

        Import using the CLI is supported using the following syntax:

        GitLab project badges can be imported using an id made up of `{project_id}:{badge_id}`, e.g.

        ```sh
        $ pulumi import gitlab:index/projectBadge:ProjectBadge foo 1:3
        ```

        :param str resource_name: The name of the resource.
        :param ProjectBadgeArgs args: The arguments to use to populate this resource's properties.
        :param pulumi.ResourceOptions opts: Options for the resource.
        """
        ...
    def __init__(__self__, resource_name: str, *args, **kwargs):
        resource_args, opts = _utilities.get_resource_args_opts(ProjectBadgeArgs, pulumi.ResourceOptions, *args, **kwargs)
        if resource_args is not None:
            __self__._internal_init(resource_name, opts, **resource_args.__dict__)
        else:
            __self__._internal_init(resource_name, *args, **kwargs)

    def _internal_init(__self__,
                 resource_name: str,
                 opts: Optional[pulumi.ResourceOptions] = None,
                 image_url: Optional[pulumi.Input[builtins.str]] = None,
                 link_url: Optional[pulumi.Input[builtins.str]] = None,
                 name: Optional[pulumi.Input[builtins.str]] = None,
                 project: Optional[pulumi.Input[builtins.str]] = None,
                 __props__=None):
        opts = pulumi.ResourceOptions.merge(_utilities.get_resource_opts_defaults(), opts)
        if not isinstance(opts, pulumi.ResourceOptions):
            raise TypeError('Expected resource options to be a ResourceOptions instance')
        if opts.id is None:
            if __props__ is not None:
                raise TypeError('__props__ is only valid when passed in combination with a valid opts.id to get an existing resource')
            __props__ = ProjectBadgeArgs.__new__(ProjectBadgeArgs)

            if image_url is None and not opts.urn:
                raise TypeError("Missing required property 'image_url'")
            __props__.__dict__["image_url"] = image_url
            if link_url is None and not opts.urn:
                raise TypeError("Missing required property 'link_url'")
            __props__.__dict__["link_url"] = link_url
            __props__.__dict__["name"] = name
            if project is None and not opts.urn:
                raise TypeError("Missing required property 'project'")
            __props__.__dict__["project"] = project
            __props__.__dict__["rendered_image_url"] = None
            __props__.__dict__["rendered_link_url"] = None
        super(ProjectBadge, __self__).__init__(
            'gitlab:index/projectBadge:ProjectBadge',
            resource_name,
            __props__,
            opts)

    @staticmethod
    def get(resource_name: str,
            id: pulumi.Input[str],
            opts: Optional[pulumi.ResourceOptions] = None,
            image_url: Optional[pulumi.Input[builtins.str]] = None,
            link_url: Optional[pulumi.Input[builtins.str]] = None,
            name: Optional[pulumi.Input[builtins.str]] = None,
            project: Optional[pulumi.Input[builtins.str]] = None,
            rendered_image_url: Optional[pulumi.Input[builtins.str]] = None,
            rendered_link_url: Optional[pulumi.Input[builtins.str]] = None) -> 'ProjectBadge':
        """
        Get an existing ProjectBadge resource's state with the given name, id, and optional extra
        properties used to qualify the lookup.

        :param str resource_name: The unique name of the resulting resource.
        :param pulumi.Input[str] id: The unique provider ID of the resource to lookup.
        :param pulumi.ResourceOptions opts: Options for the resource.
        :param pulumi.Input[builtins.str] image_url: The image url which will be presented on project overview.
        :param pulumi.Input[builtins.str] link_url: The url linked with the badge.
        :param pulumi.Input[builtins.str] name: The name of the badge.
        :param pulumi.Input[builtins.str] project: The id of the project to add the badge to.
        :param pulumi.Input[builtins.str] rendered_image_url: The image_url argument rendered (in case of use of placeholders).
        :param pulumi.Input[builtins.str] rendered_link_url: The link_url argument rendered (in case of use of placeholders).
        """
        opts = pulumi.ResourceOptions.merge(opts, pulumi.ResourceOptions(id=id))

        __props__ = _ProjectBadgeState.__new__(_ProjectBadgeState)

        __props__.__dict__["image_url"] = image_url
        __props__.__dict__["link_url"] = link_url
        __props__.__dict__["name"] = name
        __props__.__dict__["project"] = project
        __props__.__dict__["rendered_image_url"] = rendered_image_url
        __props__.__dict__["rendered_link_url"] = rendered_link_url
        return ProjectBadge(resource_name, opts=opts, __props__=__props__)

    @property
    @pulumi.getter(name="imageUrl")
    def image_url(self) -> pulumi.Output[builtins.str]:
        """
        The image url which will be presented on project overview.
        """
        return pulumi.get(self, "image_url")

    @property
    @pulumi.getter(name="linkUrl")
    def link_url(self) -> pulumi.Output[builtins.str]:
        """
        The url linked with the badge.
        """
        return pulumi.get(self, "link_url")

    @property
    @pulumi.getter
    def name(self) -> pulumi.Output[builtins.str]:
        """
        The name of the badge.
        """
        return pulumi.get(self, "name")

    @property
    @pulumi.getter
    def project(self) -> pulumi.Output[builtins.str]:
        """
        The id of the project to add the badge to.
        """
        return pulumi.get(self, "project")

    @property
    @pulumi.getter(name="renderedImageUrl")
    def rendered_image_url(self) -> pulumi.Output[builtins.str]:
        """
        The image_url argument rendered (in case of use of placeholders).
        """
        return pulumi.get(self, "rendered_image_url")

    @property
    @pulumi.getter(name="renderedLinkUrl")
    def rendered_link_url(self) -> pulumi.Output[builtins.str]:
        """
        The link_url argument rendered (in case of use of placeholders).
        """
        return pulumi.get(self, "rendered_link_url")




# coding=utf-8
# *** WARNING: this file was generated by the Pulumi Terraform Bridge (tfgen) Tool. ***
# *** Do not edit by hand unless you're certain you know what you are doing! ***

import builtins
import copy
import warnings
import sys
import pulumi
import pulumi.runtime
from typing import Any, Mapping, Optional, Sequence, Union, overload
if sys.version_info >= (3, 11):
    from typing import NotRequired, TypedDict, TypeAlias
else:
    from typing_extensions import NotRequired, TypedDict, TypeAlias
from . import _utilities

__all__ = ['ProjectApprovalRuleArgs', 'ProjectApprovalRule']

@pulumi.input_type
class ProjectApprovalRuleArgs:
    def __init__(__self__, *,
                 approvals_required: pulumi.Input[builtins.int],
                 project: pulumi.Input[builtins.str],
                 applies_to_all_protected_branches: Optional[pulumi.Input[builtins.bool]] = None,
                 disable_importing_default_any_approver_rule_on_create: Optional[pulumi.Input[builtins.bool]] = None,
                 group_ids: Optional[pulumi.Input[Sequence[pulumi.Input[builtins.int]]]] = None,
                 name: Optional[pulumi.Input[builtins.str]] = None,
                 protected_branch_ids: Optional[pulumi.Input[Sequence[pulumi.Input[builtins.int]]]] = None,
                 report_type: Optional[pulumi.Input[builtins.str]] = None,
                 rule_type: Optional[pulumi.Input[builtins.str]] = None,
                 user_ids: Optional[pulumi.Input[Sequence[pulumi.Input[builtins.int]]]] = None):
        """
        The set of arguments for constructing a ProjectApprovalRule resource.
        :param pulumi.Input[builtins.int] approvals_required: The number of approvals required for this rule.
        :param pulumi.Input[builtins.str] project: The name or id of the project to add the approval rules.
        :param pulumi.Input[builtins.bool] applies_to_all_protected_branches: Whether the rule is applied to all protected branches. If set to 'true', the value of `protected_branch_ids` is ignored. Default is 'false'.
        :param pulumi.Input[builtins.bool] disable_importing_default_any_approver_rule_on_create: When this flag is set, the default `any_approver` rule will not be imported if present.
        :param pulumi.Input[Sequence[pulumi.Input[builtins.int]]] group_ids: A list of group IDs whose members can approve of the merge request.
        :param pulumi.Input[builtins.str] name: The name of the approval rule.
        :param pulumi.Input[Sequence[pulumi.Input[builtins.int]]] protected_branch_ids: A list of protected branch IDs (not branch names) for which the rule applies.
        :param pulumi.Input[builtins.str] report_type: Report type is required when the rule_type is `report_approver`. Valid values are `code_coverage`.
        :param pulumi.Input[builtins.str] rule_type: String, defaults to 'regular'. The type of rule. `any_approver` is a pre-configured default rule with `approvals_required` at `0`. Valid values are `regular`, `any_approver`, `report_approver`.
        :param pulumi.Input[Sequence[pulumi.Input[builtins.int]]] user_ids: A list of specific User IDs to add to the list of approvers.
        """
        pulumi.set(__self__, "approvals_required", approvals_required)
        pulumi.set(__self__, "project", project)
        if applies_to_all_protected_branches is not None:
            pulumi.set(__self__, "applies_to_all_protected_branches", applies_to_all_protected_branches)
        if disable_importing_default_any_approver_rule_on_create is not None:
            pulumi.set(__self__, "disable_importing_default_any_approver_rule_on_create", disable_importing_default_any_approver_rule_on_create)
        if group_ids is not None:
            pulumi.set(__self__, "group_ids", group_ids)
        if name is not None:
            pulumi.set(__self__, "name", name)
        if protected_branch_ids is not None:
            pulumi.set(__self__, "protected_branch_ids", protected_branch_ids)
        if report_type is not None:
            pulumi.set(__self__, "report_type", report_type)
        if rule_type is not None:
            pulumi.set(__self__, "rule_type", rule_type)
        if user_ids is not None:
            pulumi.set(__self__, "user_ids", user_ids)

    @property
    @pulumi.getter(name="approvalsRequired")
    def approvals_required(self) -> pulumi.Input[builtins.int]:
        """
        The number of approvals required for this rule.
        """
        return pulumi.get(self, "approvals_required")

    @approvals_required.setter
    def approvals_required(self, value: pulumi.Input[builtins.int]):
        pulumi.set(self, "approvals_required", value)

    @property
    @pulumi.getter
    def project(self) -> pulumi.Input[builtins.str]:
        """
        The name or id of the project to add the approval rules.
        """
        return pulumi.get(self, "project")

    @project.setter
    def project(self, value: pulumi.Input[builtins.str]):
        pulumi.set(self, "project", value)

    @property
    @pulumi.getter(name="appliesToAllProtectedBranches")
    def applies_to_all_protected_branches(self) -> Optional[pulumi.Input[builtins.bool]]:
        """
        Whether the rule is applied to all protected branches. If set to 'true', the value of `protected_branch_ids` is ignored. Default is 'false'.
        """
        return pulumi.get(self, "applies_to_all_protected_branches")

    @applies_to_all_protected_branches.setter
    def applies_to_all_protected_branches(self, value: Optional[pulumi.Input[builtins.bool]]):
        pulumi.set(self, "applies_to_all_protected_branches", value)

    @property
    @pulumi.getter(name="disableImportingDefaultAnyApproverRuleOnCreate")
    def disable_importing_default_any_approver_rule_on_create(self) -> Optional[pulumi.Input[builtins.bool]]:
        """
        When this flag is set, the default `any_approver` rule will not be imported if present.
        """
        return pulumi.get(self, "disable_importing_default_any_approver_rule_on_create")

    @disable_importing_default_any_approver_rule_on_create.setter
    def disable_importing_default_any_approver_rule_on_create(self, value: Optional[pulumi.Input[builtins.bool]]):
        pulumi.set(self, "disable_importing_default_any_approver_rule_on_create", value)

    @property
    @pulumi.getter(name="groupIds")
    def group_ids(self) -> Optional[pulumi.Input[Sequence[pulumi.Input[builtins.int]]]]:
        """
        A list of group IDs whose members can approve of the merge request.
        """
        return pulumi.get(self, "group_ids")

    @group_ids.setter
    def group_ids(self, value: Optional[pulumi.Input[Sequence[pulumi.Input[builtins.int]]]]):
        pulumi.set(self, "group_ids", value)

    @property
    @pulumi.getter
    def name(self) -> Optional[pulumi.Input[builtins.str]]:
        """
        The name of the approval rule.
        """
        return pulumi.get(self, "name")

    @name.setter
    def name(self, value: Optional[pulumi.Input[builtins.str]]):
        pulumi.set(self, "name", value)

    @property
    @pulumi.getter(name="protectedBranchIds")
    def protected_branch_ids(self) -> Optional[pulumi.Input[Sequence[pulumi.Input[builtins.int]]]]:
        """
        A list of protected branch IDs (not branch names) for which the rule applies.
        """
        return pulumi.get(self, "protected_branch_ids")

    @protected_branch_ids.setter
    def protected_branch_ids(self, value: Optional[pulumi.Input[Sequence[pulumi.Input[builtins.int]]]]):
        pulumi.set(self, "protected_branch_ids", value)

    @property
    @pulumi.getter(name="reportType")
    def report_type(self) -> Optional[pulumi.Input[builtins.str]]:
        """
        Report type is required when the rule_type is `report_approver`. Valid values are `code_coverage`.
        """
        return pulumi.get(self, "report_type")

    @report_type.setter
    def report_type(self, value: Optional[pulumi.Input[builtins.str]]):
        pulumi.set(self, "report_type", value)

    @property
    @pulumi.getter(name="ruleType")
    def rule_type(self) -> Optional[pulumi.Input[builtins.str]]:
        """
        String, defaults to 'regular'. The type of rule. `any_approver` is a pre-configured default rule with `approvals_required` at `0`. Valid values are `regular`, `any_approver`, `report_approver`.
        """
        return pulumi.get(self, "rule_type")

    @rule_type.setter
    def rule_type(self, value: Optional[pulumi.Input[builtins.str]]):
        pulumi.set(self, "rule_type", value)

    @property
    @pulumi.getter(name="userIds")
    def user_ids(self) -> Optional[pulumi.Input[Sequence[pulumi.Input[builtins.int]]]]:
        """
        A list of specific User IDs to add to the list of approvers.
        """
        return pulumi.get(self, "user_ids")

    @user_ids.setter
    def user_ids(self, value: Optional[pulumi.Input[Sequence[pulumi.Input[builtins.int]]]]):
        pulumi.set(self, "user_ids", value)


@pulumi.input_type
class _ProjectApprovalRuleState:
    def __init__(__self__, *,
                 applies_to_all_protected_branches: Optional[pulumi.Input[builtins.bool]] = None,
                 approvals_required: Optional[pulumi.Input[builtins.int]] = None,
                 disable_importing_default_any_approver_rule_on_create: Optional[pulumi.Input[builtins.bool]] = None,
                 group_ids: Optional[pulumi.Input[Sequence[pulumi.Input[builtins.int]]]] = None,
                 name: Optional[pulumi.Input[builtins.str]] = None,
                 project: Optional[pulumi.Input[builtins.str]] = None,
                 protected_branch_ids: Optional[pulumi.Input[Sequence[pulumi.Input[builtins.int]]]] = None,
                 report_type: Optional[pulumi.Input[builtins.str]] = None,
                 rule_type: Optional[pulumi.Input[builtins.str]] = None,
                 user_ids: Optional[pulumi.Input[Sequence[pulumi.Input[builtins.int]]]] = None):
        """
        Input properties used for looking up and filtering ProjectApprovalRule resources.
        :param pulumi.Input[builtins.bool] applies_to_all_protected_branches: Whether the rule is applied to all protected branches. If set to 'true', the value of `protected_branch_ids` is ignored. Default is 'false'.
        :param pulumi.Input[builtins.int] approvals_required: The number of approvals required for this rule.
        :param pulumi.Input[builtins.bool] disable_importing_default_any_approver_rule_on_create: When this flag is set, the default `any_approver` rule will not be imported if present.
        :param pulumi.Input[Sequence[pulumi.Input[builtins.int]]] group_ids: A list of group IDs whose members can approve of the merge request.
        :param pulumi.Input[builtins.str] name: The name of the approval rule.
        :param pulumi.Input[builtins.str] project: The name or id of the project to add the approval rules.
        :param pulumi.Input[Sequence[pulumi.Input[builtins.int]]] protected_branch_ids: A list of protected branch IDs (not branch names) for which the rule applies.
        :param pulumi.Input[builtins.str] report_type: Report type is required when the rule_type is `report_approver`. Valid values are `code_coverage`.
        :param pulumi.Input[builtins.str] rule_type: String, defaults to 'regular'. The type of rule. `any_approver` is a pre-configured default rule with `approvals_required` at `0`. Valid values are `regular`, `any_approver`, `report_approver`.
        :param pulumi.Input[Sequence[pulumi.Input[builtins.int]]] user_ids: A list of specific User IDs to add to the list of approvers.
        """
        if applies_to_all_protected_branches is not None:
            pulumi.set(__self__, "applies_to_all_protected_branches", applies_to_all_protected_branches)
        if approvals_required is not None:
            pulumi.set(__self__, "approvals_required", approvals_required)
        if disable_importing_default_any_approver_rule_on_create is not None:
            pulumi.set(__self__, "disable_importing_default_any_approver_rule_on_create", disable_importing_default_any_approver_rule_on_create)
        if group_ids is not None:
            pulumi.set(__self__, "group_ids", group_ids)
        if name is not None:
            pulumi.set(__self__, "name", name)
        if project is not None:
            pulumi.set(__self__, "project", project)
        if protected_branch_ids is not None:
            pulumi.set(__self__, "protected_branch_ids", protected_branch_ids)
        if report_type is not None:
            pulumi.set(__self__, "report_type", report_type)
        if rule_type is not None:
            pulumi.set(__self__, "rule_type", rule_type)
        if user_ids is not None:
            pulumi.set(__self__, "user_ids", user_ids)

    @property
    @pulumi.getter(name="appliesToAllProtectedBranches")
    def applies_to_all_protected_branches(self) -> Optional[pulumi.Input[builtins.bool]]:
        """
        Whether the rule is applied to all protected branches. If set to 'true', the value of `protected_branch_ids` is ignored. Default is 'false'.
        """
        return pulumi.get(self, "applies_to_all_protected_branches")

    @applies_to_all_protected_branches.setter
    def applies_to_all_protected_branches(self, value: Optional[pulumi.Input[builtins.bool]]):
        pulumi.set(self, "applies_to_all_protected_branches", value)

    @property
    @pulumi.getter(name="approvalsRequired")
    def approvals_required(self) -> Optional[pulumi.Input[builtins.int]]:
        """
        The number of approvals required for this rule.
        """
        return pulumi.get(self, "approvals_required")

    @approvals_required.setter
    def approvals_required(self, value: Optional[pulumi.Input[builtins.int]]):
        pulumi.set(self, "approvals_required", value)

    @property
    @pulumi.getter(name="disableImportingDefaultAnyApproverRuleOnCreate")
    def disable_importing_default_any_approver_rule_on_create(self) -> Optional[pulumi.Input[builtins.bool]]:
        """
        When this flag is set, the default `any_approver` rule will not be imported if present.
        """
        return pulumi.get(self, "disable_importing_default_any_approver_rule_on_create")

    @disable_importing_default_any_approver_rule_on_create.setter
    def disable_importing_default_any_approver_rule_on_create(self, value: Optional[pulumi.Input[builtins.bool]]):
        pulumi.set(self, "disable_importing_default_any_approver_rule_on_create", value)

    @property
    @pulumi.getter(name="groupIds")
    def group_ids(self) -> Optional[pulumi.Input[Sequence[pulumi.Input[builtins.int]]]]:
        """
        A list of group IDs whose members can approve of the merge request.
        """
        return pulumi.get(self, "group_ids")

    @group_ids.setter
    def group_ids(self, value: Optional[pulumi.Input[Sequence[pulumi.Input[builtins.int]]]]):
        pulumi.set(self, "group_ids", value)

    @property
    @pulumi.getter
    def name(self) -> Optional[pulumi.Input[builtins.str]]:
        """
        The name of the approval rule.
        """
        return pulumi.get(self, "name")

    @name.setter
    def name(self, value: Optional[pulumi.Input[builtins.str]]):
        pulumi.set(self, "name", value)

    @property
    @pulumi.getter
    def project(self) -> Optional[pulumi.Input[builtins.str]]:
        """
        The name or id of the project to add the approval rules.
        """
        return pulumi.get(self, "project")

    @project.setter
    def project(self, value: Optional[pulumi.Input[builtins.str]]):
        pulumi.set(self, "project", value)

    @property
    @pulumi.getter(name="protectedBranchIds")
    def protected_branch_ids(self) -> Optional[pulumi.Input[Sequence[pulumi.Input[builtins.int]]]]:
        """
        A list of protected branch IDs (not branch names) for which the rule applies.
        """
        return pulumi.get(self, "protected_branch_ids")

    @protected_branch_ids.setter
    def protected_branch_ids(self, value: Optional[pulumi.Input[Sequence[pulumi.Input[builtins.int]]]]):
        pulumi.set(self, "protected_branch_ids", value)

    @property
    @pulumi.getter(name="reportType")
    def report_type(self) -> Optional[pulumi.Input[builtins.str]]:
        """
        Report type is required when the rule_type is `report_approver`. Valid values are `code_coverage`.
        """
        return pulumi.get(self, "report_type")

    @report_type.setter
    def report_type(self, value: Optional[pulumi.Input[builtins.str]]):
        pulumi.set(self, "report_type", value)

    @property
    @pulumi.getter(name="ruleType")
    def rule_type(self) -> Optional[pulumi.Input[builtins.str]]:
        """
        String, defaults to 'regular'. The type of rule. `any_approver` is a pre-configured default rule with `approvals_required` at `0`. Valid values are `regular`, `any_approver`, `report_approver`.
        """
        return pulumi.get(self, "rule_type")

    @rule_type.setter
    def rule_type(self, value: Optional[pulumi.Input[builtins.str]]):
        pulumi.set(self, "rule_type", value)

    @property
    @pulumi.getter(name="userIds")
    def user_ids(self) -> Optional[pulumi.Input[Sequence[pulumi.Input[builtins.int]]]]:
        """
        A list of specific User IDs to add to the list of approvers.
        """
        return pulumi.get(self, "user_ids")

    @user_ids.setter
    def user_ids(self, value: Optional[pulumi.Input[Sequence[pulumi.Input[builtins.int]]]]):
        pulumi.set(self, "user_ids", value)


class ProjectApprovalRule(pulumi.CustomResource):

    pulumi_type = "gitlab:index/projectApprovalRule:ProjectApprovalRule"

    @overload
    def __init__(__self__,
                 resource_name: str,
                 opts: Optional[pulumi.ResourceOptions] = None,
                 applies_to_all_protected_branches: Optional[pulumi.Input[builtins.bool]] = None,
                 approvals_required: Optional[pulumi.Input[builtins.int]] = None,
                 disable_importing_default_any_approver_rule_on_create: Optional[pulumi.Input[builtins.bool]] = None,
                 group_ids: Optional[pulumi.Input[Sequence[pulumi.Input[builtins.int]]]] = None,
                 name: Optional[pulumi.Input[builtins.str]] = None,
                 project: Optional[pulumi.Input[builtins.str]] = None,
                 protected_branch_ids: Optional[pulumi.Input[Sequence[pulumi.Input[builtins.int]]]] = None,
                 report_type: Optional[pulumi.Input[builtins.str]] = None,
                 rule_type: Optional[pulumi.Input[builtins.str]] = None,
                 user_ids: Optional[pulumi.Input[Sequence[pulumi.Input[builtins.int]]]] = None,
                 __props__=None):
        """
        ## Example Usage

        ## Import

        Starting in Terraform v1.5.0 you can use an import block to import `gitlab_project_approval_rule`. For example:

        terraform

        import {

          to = gitlab_project_approval_rule.example

          id = "see CLI command below for ID"

        }

        Import using the CLI is supported using the following syntax:

        GitLab project approval rules can be imported using a key composed of `<project-id>:<rule-id>`, e.g.

        ```sh
        $ pulumi import gitlab:index/projectApprovalRule:ProjectApprovalRule example "12345:6"
        ```

        :param str resource_name: The name of the resource.
        :param pulumi.ResourceOptions opts: Options for the resource.
        :param pulumi.Input[builtins.bool] applies_to_all_protected_branches: Whether the rule is applied to all protected branches. If set to 'true', the value of `protected_branch_ids` is ignored. Default is 'false'.
        :param pulumi.Input[builtins.int] approvals_required: The number of approvals required for this rule.
        :param pulumi.Input[builtins.bool] disable_importing_default_any_approver_rule_on_create: When this flag is set, the default `any_approver` rule will not be imported if present.
        :param pulumi.Input[Sequence[pulumi.Input[builtins.int]]] group_ids: A list of group IDs whose members can approve of the merge request.
        :param pulumi.Input[builtins.str] name: The name of the approval rule.
        :param pulumi.Input[builtins.str] project: The name or id of the project to add the approval rules.
        :param pulumi.Input[Sequence[pulumi.Input[builtins.int]]] protected_branch_ids: A list of protected branch IDs (not branch names) for which the rule applies.
        :param pulumi.Input[builtins.str] report_type: Report type is required when the rule_type is `report_approver`. Valid values are `code_coverage`.
        :param pulumi.Input[builtins.str] rule_type: String, defaults to 'regular'. The type of rule. `any_approver` is a pre-configured default rule with `approvals_required` at `0`. Valid values are `regular`, `any_approver`, `report_approver`.
        :param pulumi.Input[Sequence[pulumi.Input[builtins.int]]] user_ids: A list of specific User IDs to add to the list of approvers.
        """
        ...
    @overload
    def __init__(__self__,
                 resource_name: str,
                 args: ProjectApprovalRuleArgs,
                 opts: Optional[pulumi.ResourceOptions] = None):
        """
        ## Example Usage

        ## Import

        Starting in Terraform v1.5.0 you can use an import block to import `gitlab_project_approval_rule`. For example:

        terraform

        import {

          to = gitlab_project_approval_rule.example

          id = "see CLI command below for ID"

        }

        Import using the CLI is supported using the following syntax:

        GitLab project approval rules can be imported using a key composed of `<project-id>:<rule-id>`, e.g.

        ```sh
        $ pulumi import gitlab:index/projectApprovalRule:ProjectApprovalRule example "12345:6"
        ```

        :param str resource_name: The name of the resource.
        :param ProjectApprovalRuleArgs args: The arguments to use to populate this resource's properties.
        :param pulumi.ResourceOptions opts: Options for the resource.
        """
        ...
    def __init__(__self__, resource_name: str, *args, **kwargs):
        resource_args, opts = _utilities.get_resource_args_opts(ProjectApprovalRuleArgs, pulumi.ResourceOptions, *args, **kwargs)
        if resource_args is not None:
            __self__._internal_init(resource_name, opts, **resource_args.__dict__)
        else:
            __self__._internal_init(resource_name, *args, **kwargs)

    def _internal_init(__self__,
                 resource_name: str,
                 opts: Optional[pulumi.ResourceOptions] = None,
                 applies_to_all_protected_branches: Optional[pulumi.Input[builtins.bool]] = None,
                 approvals_required: Optional[pulumi.Input[builtins.int]] = None,
                 disable_importing_default_any_approver_rule_on_create: Optional[pulumi.Input[builtins.bool]] = None,
                 group_ids: Optional[pulumi.Input[Sequence[pulumi.Input[builtins.int]]]] = None,
                 name: Optional[pulumi.Input[builtins.str]] = None,
                 project: Optional[pulumi.Input[builtins.str]] = None,
                 protected_branch_ids: Optional[pulumi.Input[Sequence[pulumi.Input[builtins.int]]]] = None,
                 report_type: Optional[pulumi.Input[builtins.str]] = None,
                 rule_type: Optional[pulumi.Input[builtins.str]] = None,
                 user_ids: Optional[pulumi.Input[Sequence[pulumi.Input[builtins.int]]]] = None,
                 __props__=None):
        opts = pulumi.ResourceOptions.merge(_utilities.get_resource_opts_defaults(), opts)
        if not isinstance(opts, pulumi.ResourceOptions):
            raise TypeError('Expected resource options to be a ResourceOptions instance')
        if opts.id is None:
            if __props__ is not None:
                raise TypeError('__props__ is only valid when passed in combination with a valid opts.id to get an existing resource')
            __props__ = ProjectApprovalRuleArgs.__new__(ProjectApprovalRuleArgs)

            __props__.__dict__["applies_to_all_protected_branches"] = applies_to_all_protected_branches
            if approvals_required is None and not opts.urn:
                raise TypeError("Missing required property 'approvals_required'")
            __props__.__dict__["approvals_required"] = approvals_required
            __props__.__dict__["disable_importing_default_any_approver_rule_on_create"] = disable_importing_default_any_approver_rule_on_create
            __props__.__dict__["group_ids"] = group_ids
            __props__.__dict__["name"] = name
            if project is None and not opts.urn:
                raise TypeError("Missing required property 'project'")
            __props__.__dict__["project"] = project
            __props__.__dict__["protected_branch_ids"] = protected_branch_ids
            __props__.__dict__["report_type"] = report_type
            __props__.__dict__["rule_type"] = rule_type
            __props__.__dict__["user_ids"] = user_ids
        super(ProjectApprovalRule, __self__).__init__(
            'gitlab:index/projectApprovalRule:ProjectApprovalRule',
            resource_name,
            __props__,
            opts)

    @staticmethod
    def get(resource_name: str,
            id: pulumi.Input[str],
            opts: Optional[pulumi.ResourceOptions] = None,
            applies_to_all_protected_branches: Optional[pulumi.Input[builtins.bool]] = None,
            approvals_required: Optional[pulumi.Input[builtins.int]] = None,
            disable_importing_default_any_approver_rule_on_create: Optional[pulumi.Input[builtins.bool]] = None,
            group_ids: Optional[pulumi.Input[Sequence[pulumi.Input[builtins.int]]]] = None,
            name: Optional[pulumi.Input[builtins.str]] = None,
            project: Optional[pulumi.Input[builtins.str]] = None,
            protected_branch_ids: Optional[pulumi.Input[Sequence[pulumi.Input[builtins.int]]]] = None,
            report_type: Optional[pulumi.Input[builtins.str]] = None,
            rule_type: Optional[pulumi.Input[builtins.str]] = None,
            user_ids: Optional[pulumi.Input[Sequence[pulumi.Input[builtins.int]]]] = None) -> 'ProjectApprovalRule':
        """
        Get an existing ProjectApprovalRule resource's state with the given name, id, and optional extra
        properties used to qualify the lookup.

        :param str resource_name: The unique name of the resulting resource.
        :param pulumi.Input[str] id: The unique provider ID of the resource to lookup.
        :param pulumi.ResourceOptions opts: Options for the resource.
        :param pulumi.Input[builtins.bool] applies_to_all_protected_branches: Whether the rule is applied to all protected branches. If set to 'true', the value of `protected_branch_ids` is ignored. Default is 'false'.
        :param pulumi.Input[builtins.int] approvals_required: The number of approvals required for this rule.
        :param pulumi.Input[builtins.bool] disable_importing_default_any_approver_rule_on_create: When this flag is set, the default `any_approver` rule will not be imported if present.
        :param pulumi.Input[Sequence[pulumi.Input[builtins.int]]] group_ids: A list of group IDs whose members can approve of the merge request.
        :param pulumi.Input[builtins.str] name: The name of the approval rule.
        :param pulumi.Input[builtins.str] project: The name or id of the project to add the approval rules.
        :param pulumi.Input[Sequence[pulumi.Input[builtins.int]]] protected_branch_ids: A list of protected branch IDs (not branch names) for which the rule applies.
        :param pulumi.Input[builtins.str] report_type: Report type is required when the rule_type is `report_approver`. Valid values are `code_coverage`.
        :param pulumi.Input[builtins.str] rule_type: String, defaults to 'regular'. The type of rule. `any_approver` is a pre-configured default rule with `approvals_required` at `0`. Valid values are `regular`, `any_approver`, `report_approver`.
        :param pulumi.Input[Sequence[pulumi.Input[builtins.int]]] user_ids: A list of specific User IDs to add to the list of approvers.
        """
        opts = pulumi.ResourceOptions.merge(opts, pulumi.ResourceOptions(id=id))

        __props__ = _ProjectApprovalRuleState.__new__(_ProjectApprovalRuleState)

        __props__.__dict__["applies_to_all_protected_branches"] = applies_to_all_protected_branches
        __props__.__dict__["approvals_required"] = approvals_required
        __props__.__dict__["disable_importing_default_any_approver_rule_on_create"] = disable_importing_default_any_approver_rule_on_create
        __props__.__dict__["group_ids"] = group_ids
        __props__.__dict__["name"] = name
        __props__.__dict__["project"] = project
        __props__.__dict__["protected_branch_ids"] = protected_branch_ids
        __props__.__dict__["report_type"] = report_type
        __props__.__dict__["rule_type"] = rule_type
        __props__.__dict__["user_ids"] = user_ids
        return ProjectApprovalRule(resource_name, opts=opts, __props__=__props__)

    @property
    @pulumi.getter(name="appliesToAllProtectedBranches")
    def applies_to_all_protected_branches(self) -> pulumi.Output[Optional[builtins.bool]]:
        """
        Whether the rule is applied to all protected branches. If set to 'true', the value of `protected_branch_ids` is ignored. Default is 'false'.
        """
        return pulumi.get(self, "applies_to_all_protected_branches")

    @property
    @pulumi.getter(name="approvalsRequired")
    def approvals_required(self) -> pulumi.Output[builtins.int]:
        """
        The number of approvals required for this rule.
        """
        return pulumi.get(self, "approvals_required")

    @property
    @pulumi.getter(name="disableImportingDefaultAnyApproverRuleOnCreate")
    def disable_importing_default_any_approver_rule_on_create(self) -> pulumi.Output[Optional[builtins.bool]]:
        """
        When this flag is set, the default `any_approver` rule will not be imported if present.
        """
        return pulumi.get(self, "disable_importing_default_any_approver_rule_on_create")

    @property
    @pulumi.getter(name="groupIds")
    def group_ids(self) -> pulumi.Output[Optional[Sequence[builtins.int]]]:
        """
        A list of group IDs whose members can approve of the merge request.
        """
        return pulumi.get(self, "group_ids")

    @property
    @pulumi.getter
    def name(self) -> pulumi.Output[builtins.str]:
        """
        The name of the approval rule.
        """
        return pulumi.get(self, "name")

    @property
    @pulumi.getter
    def project(self) -> pulumi.Output[builtins.str]:
        """
        The name or id of the project to add the approval rules.
        """
        return pulumi.get(self, "project")

    @property
    @pulumi.getter(name="protectedBranchIds")
    def protected_branch_ids(self) -> pulumi.Output[Optional[Sequence[builtins.int]]]:
        """
        A list of protected branch IDs (not branch names) for which the rule applies.
        """
        return pulumi.get(self, "protected_branch_ids")

    @property
    @pulumi.getter(name="reportType")
    def report_type(self) -> pulumi.Output[builtins.str]:
        """
        Report type is required when the rule_type is `report_approver`. Valid values are `code_coverage`.
        """
        return pulumi.get(self, "report_type")

    @property
    @pulumi.getter(name="ruleType")
    def rule_type(self) -> pulumi.Output[builtins.str]:
        """
        String, defaults to 'regular'. The type of rule. `any_approver` is a pre-configured default rule with `approvals_required` at `0`. Valid values are `regular`, `any_approver`, `report_approver`.
        """
        return pulumi.get(self, "rule_type")

    @property
    @pulumi.getter(name="userIds")
    def user_ids(self) -> pulumi.Output[Optional[Sequence[builtins.int]]]:
        """
        A list of specific User IDs to add to the list of approvers.
        """
        return pulumi.get(self, "user_ids")




# coding=utf-8
# *** WARNING: this file was generated by the Pulumi Terraform Bridge (tfgen) Tool. ***
# *** Do not edit by hand unless you're certain you know what you are doing! ***

import builtins
import copy
import warnings
import sys
import pulumi
import pulumi.runtime
from typing import Any, Mapping, Optional, Sequence, Union, overload
if sys.version_info >= (3, 11):
    from typing import NotRequired, TypedDict, TypeAlias
else:
    from typing_extensions import NotRequired, TypedDict, TypeAlias
from . import _utilities
from . import outputs

__all__ = [
    'GetProjectMergeRequestResult',
    'AwaitableGetProjectMergeRequestResult',
    'get_project_merge_request',
    'get_project_merge_request_output',
]

@pulumi.output_type
class GetProjectMergeRequestResult:
    """
    A collection of values returned by getProjectMergeRequest.
    """
    def __init__(__self__, assignee=None, assignees=None, author=None, blocking_discussions_resolved=None, changes_count=None, closed_at=None, closed_by=None, created_at=None, id=None, iid=None, project=None):
        if assignee and not isinstance(assignee, dict):
            raise TypeError("Expected argument 'assignee' to be a dict")
        pulumi.set(__self__, "assignee", assignee)
        if assignees and not isinstance(assignees, list):
            raise TypeError("Expected argument 'assignees' to be a list")
        pulumi.set(__self__, "assignees", assignees)
        if author and not isinstance(author, dict):
            raise TypeError("Expected argument 'author' to be a dict")
        pulumi.set(__self__, "author", author)
        if blocking_discussions_resolved and not isinstance(blocking_discussions_resolved, bool):
            raise TypeError("Expected argument 'blocking_discussions_resolved' to be a bool")
        pulumi.set(__self__, "blocking_discussions_resolved", blocking_discussions_resolved)
        if changes_count and not isinstance(changes_count, str):
            raise TypeError("Expected argument 'changes_count' to be a str")
        pulumi.set(__self__, "changes_count", changes_count)
        if closed_at and not isinstance(closed_at, str):
            raise TypeError("Expected argument 'closed_at' to be a str")
        pulumi.set(__self__, "closed_at", closed_at)
        if closed_by and not isinstance(closed_by, dict):
            raise TypeError("Expected argument 'closed_by' to be a dict")
        pulumi.set(__self__, "closed_by", closed_by)
        if created_at and not isinstance(created_at, str):
            raise TypeError("Expected argument 'created_at' to be a str")
        pulumi.set(__self__, "created_at", created_at)
        if id and not isinstance(id, int):
            raise TypeError("Expected argument 'id' to be a int")
        pulumi.set(__self__, "id", id)
        if iid and not isinstance(iid, int):
            raise TypeError("Expected argument 'iid' to be a int")
        pulumi.set(__self__, "iid", iid)
        if project and not isinstance(project, str):
            raise TypeError("Expected argument 'project' to be a str")
        pulumi.set(__self__, "project", project)

    @property
    @pulumi.getter
    def assignee(self) -> 'outputs.GetProjectMergeRequestAssigneeResult':
        """
        First assignee of the merge request.
        """
        return pulumi.get(self, "assignee")

    @property
    @pulumi.getter
    def assignees(self) -> Sequence['outputs.GetProjectMergeRequestAssigneeResult']:
        """
        Assignees of the merge request.
        """
        return pulumi.get(self, "assignees")

    @property
    @pulumi.getter
    def author(self) -> 'outputs.GetProjectMergeRequestAuthorResult':
        """
        User who created this merge request.
        """
        return pulumi.get(self, "author")

    @property
    @pulumi.getter(name="blockingDiscussionsResolved")
    def blocking_discussions_resolved(self) -> builtins.bool:
        """
        Indicates if all discussions are resolved only if all are
        required before merge request can be merged.
        """
        return pulumi.get(self, "blocking_discussions_resolved")

    @property
    @pulumi.getter(name="changesCount")
    def changes_count(self) -> builtins.str:
        """
        Number of changes made on the merge request. Empty when the
        merge request is created, and populates asynchronously.
        """
        return pulumi.get(self, "changes_count")

    @property
    @pulumi.getter(name="closedAt")
    def closed_at(self) -> builtins.str:
        """
        Timestamp of when the merge request was closed.
        """
        return pulumi.get(self, "closed_at")

    @property
    @pulumi.getter(name="closedBy")
    def closed_by(self) -> 'outputs.GetProjectMergeRequestClosedByResult':
        """
        User who closed this merge request.
        """
        return pulumi.get(self, "closed_by")

    @property
    @pulumi.getter(name="createdAt")
    def created_at(self) -> builtins.str:
        """
        Timestamp of when the merge request was created.
        """
        return pulumi.get(self, "created_at")

    @property
    @pulumi.getter
    def id(self) -> builtins.int:
        """
        The unique instance level ID of the merge request.
        """
        return pulumi.get(self, "id")

    @property
    @pulumi.getter
    def iid(self) -> builtins.int:
        """
        The unique project level ID of the merge request.
        """
        return pulumi.get(self, "iid")

    @property
    @pulumi.getter
    def project(self) -> builtins.str:
        """
        The ID or path of the project.
        """
        return pulumi.get(self, "project")


class AwaitableGetProjectMergeRequestResult(GetProjectMergeRequestResult):
    # pylint: disable=using-constant-test
    def __await__(self):
        if False:
            yield self
        return GetProjectMergeRequestResult(
            assignee=self.assignee,
            assignees=self.assignees,
            author=self.author,
            blocking_discussions_resolved=self.blocking_discussions_resolved,
            changes_count=self.changes_count,
            closed_at=self.closed_at,
            closed_by=self.closed_by,
            created_at=self.created_at,
            id=self.id,
            iid=self.iid,
            project=self.project)


def get_project_merge_request(iid: Optional[builtins.int] = None,
                              project: Optional[builtins.str] = None,
                              opts: Optional[pulumi.InvokeOptions] = None) -> AwaitableGetProjectMergeRequestResult:
    """
    The `get_project_merge_request` data source retrieves
    information about a single merge request related to a specific project.

    **Upstream API**: [GitLab REST API docs](https://docs.gitlab.com/api/merge_requests/#get-single-mr)

    ## Example Usage

    ```python
    import pulumi
    import pulumi_gitlab as gitlab

    by_project_id = gitlab.get_project_merge_request(project="123",
        iid=456)
    by_project_name = gitlab.get_project_merge_request(project="company/group/project1",
        iid=3)
    ```


    :param builtins.int iid: The unique project level ID of the merge request.
    :param builtins.str project: The ID or path of the project.
    """
    __args__ = dict()
    __args__['iid'] = iid
    __args__['project'] = project
    opts = pulumi.InvokeOptions.merge(_utilities.get_invoke_opts_defaults(), opts)
    __ret__ = pulumi.runtime.invoke('gitlab:index/getProjectMergeRequest:getProjectMergeRequest', __args__, opts=opts, typ=GetProjectMergeRequestResult).value

    return AwaitableGetProjectMergeRequestResult(
        assignee=pulumi.get(__ret__, 'assignee'),
        assignees=pulumi.get(__ret__, 'assignees'),
        author=pulumi.get(__ret__, 'author'),
        blocking_discussions_resolved=pulumi.get(__ret__, 'blocking_discussions_resolved'),
        changes_count=pulumi.get(__ret__, 'changes_count'),
        closed_at=pulumi.get(__ret__, 'closed_at'),
        closed_by=pulumi.get(__ret__, 'closed_by'),
        created_at=pulumi.get(__ret__, 'created_at'),
        id=pulumi.get(__ret__, 'id'),
        iid=pulumi.get(__ret__, 'iid'),
        project=pulumi.get(__ret__, 'project'))
def get_project_merge_request_output(iid: Optional[pulumi.Input[builtins.int]] = None,
                                     project: Optional[pulumi.Input[builtins.str]] = None,
                                     opts: Optional[Union[pulumi.InvokeOptions, pulumi.InvokeOutputOptions]] = None) -> pulumi.Output[GetProjectMergeRequestResult]:
    """
    The `get_project_merge_request` data source retrieves
    information about a single merge request related to a specific project.

    **Upstream API**: [GitLab REST API docs](https://docs.gitlab.com/api/merge_requests/#get-single-mr)

    ## Example Usage

    ```python
    import pulumi
    import pulumi_gitlab as gitlab

    by_project_id = gitlab.get_project_merge_request(project="123",
        iid=456)
    by_project_name = gitlab.get_project_merge_request(project="company/group/project1",
        iid=3)
    ```


    :param builtins.int iid: The unique project level ID of the merge request.
    :param builtins.str project: The ID or path of the project.
    """
    __args__ = dict()
    __args__['iid'] = iid
    __args__['project'] = project
    opts = pulumi.InvokeOutputOptions.merge(_utilities.get_invoke_opts_defaults(), opts)
    __ret__ = pulumi.runtime.invoke_output('gitlab:index/getProjectMergeRequest:getProjectMergeRequest', __args__, opts=opts, typ=GetProjectMergeRequestResult)
    return __ret__.apply(lambda __response__: GetProjectMergeRequestResult(
        assignee=pulumi.get(__response__, 'assignee'),
        assignees=pulumi.get(__response__, 'assignees'),
        author=pulumi.get(__response__, 'author'),
        blocking_discussions_resolved=pulumi.get(__response__, 'blocking_discussions_resolved'),
        changes_count=pulumi.get(__response__, 'changes_count'),
        closed_at=pulumi.get(__response__, 'closed_at'),
        closed_by=pulumi.get(__response__, 'closed_by'),
        created_at=pulumi.get(__response__, 'created_at'),
        id=pulumi.get(__response__, 'id'),
        iid=pulumi.get(__response__, 'iid'),
        project=pulumi.get(__response__, 'project')))
